@@ -617,8 +617,8 @@ class FitMod3D(Model3D):
         libBB.Galfit_writeModel(self._mod,self._opts['norm'][0].encode('utf-8'),False)
         
         # Loading final rings
-        try: self.bfit = np.genfromtxt(op['outfolder'][0]+"/ringlog2.txt")
-        except: self.bfit = np.genfromtxt(op['outfolder'][0]+"/ringlog1.txt")
+        try: self.bfit = np.genfromtxt(op['outfolder'][0]+"/rings_final2.txt")
+        except: self.bfit = np.genfromtxt(op['outfolder'][0]+"/rings_final1.txt")
         
         # Loading final model
         for fname in os.listdir(op['outfolder'][0]):
@@ -683,8 +683,8 @@ class Search(Task):
                        "threshold"   : [0, np.float32, "Flux threshold for a detection"],
                        "adjacent"    : [True, np.bool, "Use the adjacent criterion for objects merger?" ],
                        "thrspatial"  : [-1, np.int, "Maximum spatial separation between objects"],
-                       "thrvelocity" : [-1, np.int, "Maximum channels separation between objects"],
-                       "minchannels" : [-1, np.int, "Minimum channels to make an object"],
+                       "thrvelocity" : [2,  np.int, "Maximum channels separation between objects"],
+                       "minchannels" : [2,  np.int, "Minimum channels to make an object"],
                        "minpixels"   : [-1, np.int, "Minimum pixels required in an object"],
                        "minvoxels"   : [-1, np.int, "Minimum voxels required in an object"],
                        "maxchannels" : [-1, np.int, "Maximum channels to accept an object"],
@@ -800,7 +800,7 @@ class FitMod2D(Task):
         self._mod = libBB.Fit2D_new(self.inp._cube,self._inri._rings,op['mask'][0].encode('utf-8'),op['free'][0].encode('utf-8'),\
                                     op['side'][0].encode('utf-8'),op['wfunc'][0],int(threads))
         libBB.Fit2D_compute(self._mod)
-        libBB.Fit2D_write(self._mod,'rings.txt'.encode('utf-8'))
+        libBB.Fit2D_write(self._mod,self.inp._cube,'rings.txt'.encode('utf-8'))
         a = np.genfromtxt('rings.txt') 
         try: os.remove('rings.txt')
         except: pass
@@ -826,7 +826,7 @@ class FitMod2D(Task):
         """ Write the best-fit ring model to a text file """
         if self._inri is None or self._mod is None: 
             raise ValueError("%s ERROR: you need to compute the model with compute (...) before calling writeto()."%self.taskname)
-        libBB.Fit2D_write(self._mod,filename.encode('utf-8'))
+        libBB.Fit2D_write(self._mod,self.inp._cube,filename.encode('utf-8'))
 
 
 
@@ -918,4 +918,60 @@ class Ellprof(Task):
         """ Write the profiles to a text file """
         if self._inri is None or self._mod is None: 
             raise ValueError("%s ERROR: you need to compute the profiles with compute() before calling writeto()."%self.taskname)
-        libBB.Ellprof_write(self._mod,filename.encode('utf-8'))
\ No newline at end of file
+        libBB.Ellprof_write(self._mod,filename.encode('utf-8'))
+
+
+
+class SpectralSmoothing(Task):
+    """ Spectral smoothing of a datacube with a filtering window.
+    
+    Args:
+      fitsname (str): FITS file to fit with a 2D tilted-ring model
+    """
+    def __init__(self,fitsname):
+        super(SpectralSmoothing,self).__init__(fitsname=fitsname)
+        self.taskname = 'SPECTRALSMOOTHING'
+        # Pointer to the C++ Hanning class
+        self._ptr = None  
+    
+    
+    def __del__(self):
+        if self._ptr: libBB.SpectralSmooth3D_delete(self._ptr)
+        
+
+    def smooth(self,window_type='hanning',window_size=3,threads=1):
+        """ Compute an smoothed 3D array 
+
+        Args:
+          window_type (str): type of filtering window (hanning,bartlett,welch,flattop,boxcar,blackman)
+          window_size (int): width of the window (it has to be an odd number)
+          threads (int):     number of cpus to use for computation.
+        
+        Returns:
+        a 3D spectrally-smoothed array 
+        """
+        known_windows = ['HANNING','HANNING2','BOXCAR','BARTLETT','WELCH','BLACKMAN','FLATTOP']
+        self.__del__()
+        if window_size % 2 == 0:
+            raise ValueError("%s ERROR: window_size must be an odd number."%self.taskname)
+        if window_type.upper() not in known_windows:
+            raise ValueError("%s ERROR: window_type could only be HANNING, HANNING2, \
+                             BARTLETT, WELCH, BLACKMAN or FLATTOP"%(self.taskname))
+            
+        # Initializing SmoothSpectral smooth class
+        self._ptr = libBB.SpectralSmooth3D_new(window_type.encode('utf-8'),int(window_size))
+        # Performing Hanning smoothing class
+        libBB.SpectralSmooth3D_compute(self._ptr,self.inp._cube,int(threads))
+        return reshapePointer(libBB.SpectralSmooth3D_array(self._ptr),self.inp.dim[::-1])
+        
+        
+    def writeto(self,filename="",average=False):
+        """ Write the spectrally-smoothed 3D array to a FITS file 
+        
+        Args:
+          filename (str): name of output FITS file
+          average (bool): if true, average over window_size channels
+        """
+        if self._ptr is None: 
+            raise ValueError("%s ERROR: you need to Hanning smooth the data with compute() before calling writeto()."%self.taskname)
+        libBB.SpectralSmooth3D_write(self._ptr,self.inp._cube,filename.encode('utf-8'),average)
\ No newline at end of file
