@@ -22,16 +22,20 @@
 -----------------------------------------------------------------------*/
 
 #include <iostream>
+#include <iomanip>
 #include <cstring>
+#include <sstream>
 #include <cmath>
 #include <cctype>
 #include <vector>
+#include <algorithm>
 #include <fitsio.h>
 #include <wcslib/wcs.h>
 #include <wcslib/wcsunits.h>
 #include <wcslib/wcshdr.h>
 #include <wcslib/wcsfix.h>
 #include <Arrays/header.hh>
+#include <Utilities/utils.hh>
 
 Header::Header () {
     
@@ -53,9 +57,9 @@ Header::Header () {
 Header::~Header () {
     
     if (pointAllocated) {
-        delete [] dimAxes;              
-        delete [] crpix;                    
-        delete [] crval;                    
+        delete [] dimAxes;
+        delete [] crpix;
+        delete [] crval;
         delete [] cdelt;
         delete [] ctype;
         delete [] cunit;
@@ -128,8 +132,6 @@ Header& Header::operator=(const Header& h) {
     this->warning   = h.warning;
     
 //    this->wcs = h.wcs;
-    this->wcsIsGood = h.wcsIsGood;
-    if (this->wcsIsGood) {
 #pragma omp critical (wcs_header)
 {
     this->wcs = new struct wcsprm;
@@ -138,9 +140,9 @@ Header& Header::operator=(const Header& h) {
     wcscopy(true, h.wcs, this->wcs);
     wcsset(this->wcs);
     this->nwcs      = h.nwcs;
+    this->wcsIsGood = h.wcsIsGood;
 }
-    }
-    
+
     for (unsigned int i=0; i<h.keys.size(); i++)
         this->keys.push_back(h.keys[i]); 
             
@@ -170,6 +172,7 @@ void Header::setNumAx (int size){
     ctype   = new std::string[numAxes];
     pointAllocated = true;
 
+    wcsini(true, numAxes, wcs);
 }
 
 
@@ -185,7 +188,7 @@ void Header::calcArea () {
 
 bool Header::header_read (std::string fname) {
         
-    fitsfile *fptr;                                 
+    fitsfile *fptr;
     int status=0, nfound;
     char comment[72];
     
@@ -258,7 +261,7 @@ bool Header::header_read (std::string fname) {
         Warning("HEADER ERROR: CRPIXs keywords not found.");
         return false;
     }
-    
+        
     status=0;
     fits_read_keys_dbl (fptr, "CRVAL", 1, numAxes, crval, &nfound, &status);
     if (nfound==0) {
@@ -301,7 +304,7 @@ bool Header::header_read (std::string fname) {
         if (numAxes>0) cunit[0] = "DEGREE";
         if (numAxes>1) cunit[1] = "DEGREE";
         if (numAxes>2) {
-            if (ctype[2]=="FREQ" || ctype[2]=="freq" || ctype[2]=="Freq") {
+            if (makelower(ctype[2]).find("freq")!=std::string::npos) {
                 cunit[2] = "HZ";
                 Warning("HEADER WARNING: CUNITs keywords not found. Assuming [DEGREE,DEGREE,HZ]");
             }
@@ -314,6 +317,18 @@ bool Header::header_read (std::string fname) {
     else {
         for (int i=0; i<numAxes; i++) {
             cunit[i] = Cunit[i];
+            // Fixing "s-1" problem in CUNIT3
+            if (cunit[i].find("km s-1")!=std::string::npos) cunit[i] = "km/s";
+            if (cunit[i].find("m s-1")!=std::string::npos) cunit[i] = "m/s";
+            if (cunit[i]=="") {
+                cunit[i] = "deg";
+                if (i==2) cunit[i] = "km/s";
+                if (i==3) cunit[i] = "stokes";
+                std::stringstream toprint;
+                toprint << "HEADER WARNING: CUNIT" << i+1 << " keywords not found. Assuming " << cunit[i]; 
+                Warning(toprint.str());
+            }
+            
         }
     }
 
@@ -339,6 +354,10 @@ bool Header::header_read (std::string fname) {
         bunit = "NONE";
     }
     else bunit = Bunit;
+
+    if (makelower(bunit).find("beam-1")!=std::string::npos &&
+        makelower(bunit).find("jy")!=std::string::npos)
+            bunit = "JY/BEAM";
     
     status=0;
     if (fits_read_key_str (fptr, "BTYPE", Btype, comment, &status)) {
@@ -378,27 +397,30 @@ bool Header::header_read (std::string fname) {
     }
     
     status=0;
-    if (fits_read_key_dbl (fptr, "FREQ0", &freq0, comment, &status)) {
+    if (fits_read_key_dbl (fptr, "FREQR", &freq0, comment, &status)) {
         status=0;
         if (fits_read_key_dbl (fptr, "RESTFREQ", &freq0, comment, &status)) {
             status=0;
-            if (fits_read_key_dbl (fptr, "RESTFRQ", &freq0, comment, &status)) {
-                if (dunit3=="NONE" || drval3==0 || (cunit[2]!="HZ" && cunit[2]!="hz" &&
-                    cunit[2]!="MHZ" && cunit[2]!="Mhz" && cunit[2]!="mhz" &&
-                    cunit[2]!="GHZ" && cunit[2]!="Ghz" && cunit[2]!="ghz" &&
-                    dunit3!="KM/S" && dunit3!="km/s" && dunit3!="M/S"  && dunit3!="m/s")) {
-                    Warning("HEADER WARNING: FREQ0-RESTFREQ keyword not found. Assuming 1.4204057 GHz.");
-                    freq0 = 0.1420405751786E10;
-                }
-                else {
-                    double drval3ms=0.;
-                    double crval3hz=0.;
-                    if (dunit3=="KM/S" || dunit3=="km/s") drval3ms=drval3*1000;
-                    else if (dunit3=="M/S" || dunit3=="m/s") drval3ms=drval3;
-                    if (cunit[2]=="HZ" || cunit[2]=="hz") crval3hz = crval[2];
-                    else if (cunit[2]=="MHZ" || cunit[2]=="Mhz" || cunit[2]=="mhz") crval3hz=crval[2]*1.E06;
-                    else if (cunit[2]=="GHZ" || cunit[2]=="Ghz" || cunit[2]=="ghz") crval3hz=crval[2]*1.E09;
-                    freq0 = crval3hz*sqrt((299792458.+drval3ms)/(299792458.-drval3ms));
+            if (fits_read_key_dbl (fptr, "RESTFREQ", &freq0, comment, &status)) {
+                status=0;
+                if (fits_read_key_dbl (fptr, "RESTFRQ", &freq0, comment, &status)) {
+                    if (dunit3=="NONE" || drval3==0 || (cunit[2]!="HZ" && cunit[2]!="hz" &&
+                                                        cunit[2]!="MHZ" && cunit[2]!="Mhz" && cunit[2]!="mhz" &&
+                                                        cunit[2]!="GHZ" && cunit[2]!="Ghz" && cunit[2]!="ghz" &&
+                                                        dunit3!="KM/S" && dunit3!="km/s" && dunit3!="M/S"  && dunit3!="m/s")) {
+                        Warning("HEADER WARNING: FREQ0-RESTFREQ keyword not found. Assuming 1.4204057 GHz.");
+                        freq0 = 0.1420405751786E10;
+                    }
+                    else {
+                        double drval3ms=0.;
+                        double crval3hz=0.;
+                        if (dunit3=="KM/S" || dunit3=="km/s") drval3ms=drval3*1000;
+                        else if (dunit3=="M/S" || dunit3=="m/s") drval3ms=drval3;
+                        if (cunit[2]=="HZ" || cunit[2]=="hz") crval3hz = crval[2];
+                        else if (cunit[2]=="MHZ" || cunit[2]=="Mhz" || cunit[2]=="mhz") crval3hz=crval[2]*1.E06;
+                        else if (cunit[2]=="GHZ" || cunit[2]=="Ghz" || cunit[2]=="ghz") crval3hz=crval[2]*1.E09;
+                        freq0 = crval3hz*sqrt((299792458.+drval3ms)/(299792458.-drval3ms));
+                    }
                 }
             }
         }
@@ -422,7 +444,7 @@ bool Header::header_read (std::string fname) {
         if (object[i]=='/' || object[i]=='\\') object.replace(i,1, "-");
         if (object[i]=='(' || object[i]==')')  object.replace(i,1, "-");
         if (isspace(object[i])) object.replace(i,1, "_");
-    }
+    }    
 
     status = 0;
     if (fits_read_key_str (fptr, "TELESCOP", Tel, comment, &status)) {
@@ -494,14 +516,15 @@ bool Header::header_read (std::string fname) {
                 bpa = strtod(pEnd,NULL);
             }
         }
-        if (bmaj!=0 && bmin!=0) {
+        if (bmaj!=0 && bmin!=0 && warning) {
+            std::cout << std::setprecision(5);
             std::cout << "\n--------> WARNING: beam information found in HISTORY keywords: <--------\n"
                       << " BMAJ = " << bmaj << " " << cunit[0]
                       << "  BMIN = " << bmin << " " << cunit[0]
                       << "  BPA = "  << bpa  << " DEGREE\n";
             std::cout << " It is heartly recommended to check these values before going on!!\n\n";
         }
-    }
+    }    
     
     char bm[30];
     status=0;
@@ -524,7 +547,6 @@ bool Header::header_read (std::string fname) {
     if (clbmin!=0) bmin = clbmin/3600.;
 
 
-
     int noComments = 1;     // fits_hdr2str will ignore COMMENT, HISTORY etc
     int nExc = 0;
     char *hdr=0;
@@ -553,11 +575,11 @@ bool Header::header_read (std::string fname) {
     // Re-do the corrections to account for things like NCP projections
     status = wcsfix(1, (const int*)dimAxes, wcs, stat);
 
-   char stype[5],scode[5],sname[22],units[8],ptype,xtype;
-   int restreq;
-
-   status = spctyp(wcs->ctype[wcs->spec],stype,scode,sname,units,&ptype,&xtype,&restreq);
+    char stype[5],scode[5],sname[22],units[8],ptype,xtype;
+    int restreq;
+    status = spctyp(wcs->ctype[wcs->spec],stype,scode,sname,units,&ptype,&xtype,&restreq);
 
+    if((wcs->lng!=-1) && (wcs->lat!=-1)) wcsIsGood = true;
 
     // Close the FITS File
     status=0;
@@ -593,8 +615,8 @@ void Header::headwrite_3d (fitsfile *fptr, bool fullHead) {
     fits_update_key_dbl(fptr, "CDELT3", cdelt[2], 10, com, &status);
     fits_update_key_str(fptr, "CTYPE3", ctype[2].c_str(), com, &status);
     fits_update_key_str(fptr, "CUNIT3", cunit[2].c_str(), com, &status);
-    if (drval3!=0) fits_update_key_dbl(fptr, "DRVAL3", drval3, 10, com, &status);
-    if (dunit3!="NONE") fits_update_key_str(fptr, "DUNIT3", dunit3.c_str(), com, &status);
+    //if (drval3!=0) fits_update_key_dbl(fptr, "DRVAL3", drval3, 10, com, &status);
+    //if (dunit3!="NONE") fits_update_key_str(fptr, "DUNIT3", dunit3.c_str(), com, &status);
     fits_update_key_str(fptr, "BUNIT", bunit.c_str(), com, &status);
     
     if (bmaj!=0) fits_update_key_dbl(fptr, "BMAJ", bmaj, 10, com, &status);
@@ -698,116 +720,201 @@ void Header::headwrite_2d (fitsfile *fptr, bool fullHead) {
     fits_report_error(stderr, status);  
 }
 
+
+void Header::updateWCS() {
+
+    // Define the wcsprm structure given the entries in the Header
+
+    //wcsvfree(&nwcs,&wcs);
+    //wcsini(true,numAxes,wcs);
+    //wcs->flag = 0;
+    for (short i=0; i<numAxes; i++) {
+        wcs->crpix[i] = crpix[i];
+        wcs->crval[i] = crval[i];
+        wcs->cdelt[i] = cdelt[i];
+        strcpy(&wcs->cunit[i][0],cunit[i].c_str());
+        strcpy(&wcs->ctype[i][0],ctype[i].c_str());
+    }
+
+    int stat[NWCSFIX];
+    int status = wcsfix(1, (const int*)dimAxes, wcs, stat);
+    status = wcsset(wcs);
+    status = wcsfix(1, (const int*)dimAxes, wcs, stat);
+
+    if((wcs->lng!=-1) && (wcs->lat!=-1)) wcsIsGood = true;
+
+}
+
+
+int Header::wcsToPix(const double *world, double *pix, size_t npts) {
+
+    ///  @details
+    ///   Uses wcs to convert the three-dimensional world coordinate position
+    ///   referenced by world to pixel coordinates, which are placed in the
+    ///   array pix[].
+    ///   world is assumed to hold the positions of npts points.
+    ///   Offsets the pixel values by 1 to account for the C arrays being
+    ///   indexed to 0.
+    ///
+    /// \param wcs The wcsprm struct containing the WCS information.
+    /// \param world The array of world coordinates.
+    /// \param pix The returned array of pixel coordinates.
+    /// \param npts The number of distinct pixels in the arrays.
+
+    int naxis=wcs->naxis,status=0;
+    int specAxis = wcs->spec;
+    if(specAxis<0) specAxis=2;
+    if(specAxis>=naxis) specAxis = naxis-1;
+
+    // Test to see if there are other axes present, eg. stokes
+    if(wcs->naxis>naxis) naxis = wcs->naxis;
+
+    // Add entries for any other axes that are present, keeping the
+    //   order of pixel positions the same
+    double *tempworld = new double[naxis*npts];
+    for(int pt=0;pt<npts;pt++){
+        for(int axis=0;axis<naxis;axis++)
+            tempworld[pt*naxis+axis] = wcs->crval[axis];
+        tempworld[pt*naxis + wcs->lng]  = world[pt*3+0];
+        tempworld[pt*naxis + wcs->lat]  = world[pt*3+1];
+        tempworld[pt*naxis + specAxis] = world[pt*3+2];
+    }
+
+    int    *stat   = new int[npts];
+    double *temppix = new double[naxis*npts];
+    double *imgcrd = new double[naxis*npts];
+    double *phi    = new double[npts];
+    double *theta  = new double[npts];
+    status=wcss2p(wcs,npts,naxis,tempworld,phi,theta,
+                  imgcrd,temppix,stat);
+    if(status>0){
+        std::cerr << "\nCannot convert to wcs. WCS error code = " << status
+                  << ": stat="<<stat[0] << " : " << wcs_errmsg[status] << std::endl;
+    }
+    else{
+        // correct from 1-indexed to 0-indexed pixel array
+        //  and return just the spatial/velocity information,
+        //  keeping the order of the pixel positions the same.
+        for(int pt=0;pt<npts;pt++){
+            pix[pt*naxis + 0] = temppix[pt*naxis + wcs->lng] - 1.;
+            pix[pt*naxis + 1] = temppix[pt*naxis + wcs->lat] - 1.;
+            pix[pt*naxis + 2] = temppix[pt*naxis + specAxis] - 1.;
+        }
+    }
+
+    delete [] stat;
+    delete [] imgcrd;
+    delete [] temppix;
+    delete [] phi;
+    delete [] theta;
+    delete [] tempworld;
+    return status;
+}
+
+
+int Header::pixToWCS(const double *pix, double *world, size_t npts) {
+
+    ///   Uses wcs to convert the three-dimensional pixel positions referenced
+    ///   by pix to world coordinates, which are placed in the array world[].
+    ///   pix is assumed to hold the positions of npts points.
+    ///   Offsets these pixel values by 1 to account for the C arrays being
+    ///   indexed to 0.
+    ///
+    ///   \param pix The array of pixel coordinates.
+    ///   \param world The returned array of world coordinates.
+    ///   \param npts The number of distinct pixels in the arrays.
+
+    int naxis=wcs->naxis,status;
+    int specAxis = wcs->spec;
+    if(specAxis<0) specAxis=2;
+    if(specAxis>=naxis) specAxis = naxis-1;
+
+    // correct from 0-indexed to 1-indexed pixel array
+    // Add entries for any other axes that are present,
+    // keeping the order of pixel positions the same
+
+    double *newpix = new double[naxis*npts];
+    for(int pt=0;pt<npts;pt++){
+      for(int i=0;i<naxis;i++) newpix[pt*naxis+i] = 1.;
+      newpix[pt*naxis+wcs->lng]  += pix[pt*3+0];
+      newpix[pt*naxis+wcs->lat]  += pix[pt*3+1];
+      newpix[pt*naxis+specAxis] += pix[pt*3+2];
+    }
+
+    int    *stat      = new int[npts];
+    double *imgcrd    = new double[naxis*npts];
+    double *tempworld = new double[naxis*npts];
+    double *phi       = new double[npts];
+    double *theta     = new double[npts];
+    status=wcsp2s(wcs, npts, naxis, newpix, imgcrd, phi, theta, tempworld, stat);
+
+    if(status>0){
+        std::cerr << "\nCannot convert to wcs. WCS error code = " << status
+                  << ": stat="<<stat[0] << " : " << wcs_errmsg[status] << std::endl;
+    }
+    else {
+        // return just the spatial/velocity information, keeping the
+        // order of the pixel positions the same.
+        for(int pt=0;pt<npts;pt++){
+            world[pt*3+0] = tempworld[pt*naxis + wcs->lng];
+            world[pt*3+1] = tempworld[pt*naxis + wcs->lat];
+            world[pt*3+2] = tempworld[pt*naxis + specAxis];
+        }
+    }
+
+    delete [] stat;
+    delete [] imgcrd;
+    delete [] tempworld;
+    delete [] phi;
+    delete [] theta;
+    delete [] newpix;
+    return status;
+
+}
+
+
 template <class T>
 bool Header::read_keyword(std::string keyword, T &key, bool err) {
     
-    std::cout << "HEADER ERROR: unknown type of keyword "<< keyword << std::endl;
-    return false;
-}
-
-template <>
-bool Header::read_keyword<int>(std::string keyword, int &key, bool err) {
-    
-    fitsfile *fptr;                                 
-    int status=0;
-    char comment[72];   
- 
-    if (fits_open_file(&fptr, fitsname.c_str(), READONLY, &status)) {
-        fits_report_error(stderr, status);
-        return false;
-    }   
-    status = 0;
-    long dum = long(key);
-    if (fits_read_key_lng (fptr, keyword.c_str(), &dum, comment, &status)) {
-        if (err) fits_report_error(stderr, status);
+    int datatype=-1;
+    if (std::is_same<T, int>::value) datatype=TINT;
+    else if (std::is_same<T, long>::value) datatype=TLONG;
+    else if (std::is_same<T, float>::value) datatype=TFLOAT;
+    else if (std::is_same<T, double>::value) datatype=TDOUBLE;
+    else {
+        std::cerr << "Error: unknown type of keyword "<< keyword << std::endl;
         return false;
     }
-    key=dum;
-    
-    status=0;
-    if (fits_close_file(fptr, &status))
-        fits_report_error(stderr, status);
-    
-    return true;
-    
-}
 
-template <>
-bool Header::read_keyword<long>(std::string keyword, long &key, bool err) {
-    
-    fitsfile *fptr;                                 
+    fitsfile *fptr;
     int status=0;
-    char comment[72];   
- 
+    char comment[72];
+
     if (fits_open_file(&fptr, fitsname.c_str(), READONLY, &status)) {
         fits_report_error(stderr, status);
         return false;
-    }   
+    }
     status = 0;
-    if (fits_read_key_lng (fptr, keyword.c_str(), &key, comment, &status)) {
+    if (fits_read_key(fptr, datatype, keyword.c_str(), &key, comment, &status)) {
         if (err) fits_report_error(stderr, status);
         return false;
     }
     status = 0;
     if (fits_close_file(fptr, &status))
         fits_report_error(stderr, status);
-    
-    return true;
-    
-}
 
-template <>
-bool Header::read_keyword<float>(std::string keyword, float &key, bool err) {
-    
-    fitsfile *fptr;                                 
-    int status=0;
-    char comment[72];   
- 
-    if (fits_open_file(&fptr, fitsname.c_str(), READONLY, &status)) {
-        fits_report_error(stderr, status);
-        return false;
-    }   
-    status = 0;
-    if (fits_read_key_flt (fptr, keyword.c_str(), &key, comment, &status)) {
-        if (err) fits_report_error(stderr, status);
-        return false;
-    }
-    status = 0;
-    if (fits_close_file(fptr, &status))
-        fits_report_error(stderr, status);
-    
     return true;
-    
 }
+template bool Header::read_keyword(std::string,int&,bool);
+template bool Header::read_keyword(std::string,long&,bool);
+template bool Header::read_keyword(std::string,float&,bool);
+template bool Header::read_keyword(std::string,double&,bool);
 
-template <>
-bool Header::read_keyword<double>(std::string keyword, double &key, bool err) {
-    
-    fitsfile *fptr;                                 
-    int status=0;
-    char comment[72];   
- 
-    if (fits_open_file(&fptr, fitsname.c_str(), READONLY, &status)) {
-        fits_report_error(stderr, status);
-        return false;
-    }   
-    status = 0;
-    if (fits_read_key_dbl (fptr, keyword.c_str(), &key, comment, &status)) {
-        if (err) fits_report_error(stderr, status);
-        return false;
-    }
-    status = 0;
-    if (fits_close_file(fptr, &status))
-        fits_report_error(stderr, status);
-    
-    return true;
-    
-}
 
 template <>
 bool Header::read_keyword<std::string>(std::string keyword, std::string &key, bool err) {
     
-    fitsfile *fptr;                                 
+    fitsfile *fptr;
     int status=0;
     char comment[72];
         
@@ -833,27 +940,3 @@ bool Header::read_keyword<std::string>(std::string keyword, std::string &key, bo
     return true;
 }
 
-template <>
-bool Header::read_keyword<char*>(std::string keyword, char* &key, bool err) {
-    
-    fitsfile *fptr;                                 
-    int status=0;
-    char comment[72];
-        
-    if (fits_open_file(&fptr, fitsname.c_str(), READONLY, &status)) {
-        fits_report_error(stderr, status);
-        return false;
-    }
-    
-    status = 0;
-    if (fits_read_key_str (fptr, keyword.c_str(), key, comment, &status)) {
-        if (err) fits_report_error(stderr, status);
-        return false;
-    }
-    
-    status = 0;
-    if (fits_close_file(fptr, &status))
-        fits_report_error(stderr, status);
-    
-    return true;
-}
