@@ -30,8 +30,7 @@
 #include "../Utilities/utils.hh"
 
 
-void BBaroloWindow::on_FitslineEdit_editingFinished()
-{
+void BBaroloWindow::on_FitslineEdit_editingFinished() {
 
     QString filename = ui->FitslineEdit->text().trimmed();
     static QString lastFilename = "";
@@ -111,8 +110,9 @@ void BBaroloWindow::on_FitslineEdit_editingFinished()
     lastFilename = filename;
 }
 
-void BBaroloWindow::on_FitspushButton_clicked()
-{
+
+void BBaroloWindow::on_FitspushButton_clicked() {
+    
     QString filename = QFileDialog::getOpenFileName(this, tr("Open FITSFILE"), QDir::currentPath(), tr("FITS files (*.fit *.fits *.FITS *.FIT);; All files (*.*)"), 0, QFileDialog::DontUseNativeDialog);
     if (!filename.isNull()) {
         ui->FitslineEdit->setText(filename);
@@ -120,8 +120,9 @@ void BBaroloWindow::on_FitspushButton_clicked()
     }
 }
 
-void BBaroloWindow::on_ParampushButton_clicked()
-{
+
+void BBaroloWindow::on_ParampushButton_clicked() {
+    
     QString filename = QFileDialog::getOpenFileName(this, tr("Open parameter file"), QDir::currentPath(), tr("Parameter files (*.par *.txt *.dat)"), 0, QFileDialog::DontUseNativeDialog);
     if( !filename.isNull()) {
         resetGUI();
@@ -130,8 +131,9 @@ void BBaroloWindow::on_ParampushButton_clicked()
     }
 }
 
-void BBaroloWindow::on_OutfolderpushButton_clicked()
-{
+
+void BBaroloWindow::on_OutfolderpushButton_clicked() {
+    
     QString outdir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),QDir::currentPath(),QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
     if(!outdir.isNull()) {
         outdir.append("/");
@@ -140,8 +142,9 @@ void BBaroloWindow::on_OutfolderpushButton_clicked()
     }
 }
 
-void BBaroloWindow::updateText()
-{
+
+void BBaroloWindow::updateText() {
+    
     QString appendText(proc->readAll());
     ui->LogtextEdit->insertPlainText (appendText);
     ui->LogtextEdit->moveCursor (QTextCursor::End);
@@ -149,8 +152,9 @@ void BBaroloWindow::updateText()
     if (get3DFitFlag()) plotParameters();
 }
 
-void BBaroloWindow::updateExit()
-{
+
+void BBaroloWindow::updateExit() {
+    
     ui->RunpushButton->setEnabled(true);
     enable_All();
     
@@ -168,8 +172,9 @@ void BBaroloWindow::updateExit()
 
 }
 
-void BBaroloWindow::on_KillpushButton_clicked()
-{
+
+void BBaroloWindow::on_KillpushButton_clicked(){ 
+    
     if (proc->state()==2) {
         proc->terminate();
         QTimer::singleShot(2000, proc, SLOT(kill()));
@@ -178,6 +183,7 @@ void BBaroloWindow::on_KillpushButton_clicked()
     }
 }
 
+
 void BBaroloWindow::on_RunpushButton_clicked() {
 
     ui->listWidget->setCurrentRow(ui->listWidget->model()->rowCount()-2);
@@ -195,8 +201,8 @@ void BBaroloWindow::on_RunpushButton_clicked() {
     out_path = QString::fromStdString(outfolder);
     mkdirp (out_path.toStdString().c_str());
 
-    QFile::remove(out_path+"ringlog1.txt");
-    QFile::remove(out_path+"ringlog2.txt");
+    QFile::remove(out_path+"rings_final1.txt");
+    QFile::remove(out_path+"rings_final2.txt");
 
     QString paramfile = out_path+obj+".par";
     writeParamFile(paramfile);
@@ -220,10 +226,11 @@ void BBaroloWindow::on_RunpushButton_clicked() {
     
 }
 
+
 void BBaroloWindow::readParamFromFile(std::string filein) {
 
     Param *par = new Param;
-    par->readParams(filein);
+    par->readParamFile(filein);
 
     QString filename = QString::fromStdString(par->getImageFile());
     bool isBox = filename.contains("[") && filename.contains("]");
@@ -305,8 +312,8 @@ void BBaroloWindow::readParamFromFile(std::string filein) {
 
         ui->SecondstagecheckBox->setChecked(p.TWOSTAGE);
         if (p.TWOSTAGE) {
-            if (makelower(p.POLYN)=="bezier") ui->PolynspinBox->setValue(-1);
-            else ui->PolynspinBox->setValue(atoi(p.POLYN.c_str()));
+            if (makelower(p.REGTYPE)=="bezier") ui->PolynspinBox->setValue(-1);
+            else ui->PolynspinBox->setValue(atoi(p.REGTYPE.c_str()));
         }
 
         if (p.flagERRORS) ui->ErrorsradioButton->setChecked(true);
@@ -390,10 +397,10 @@ void BBaroloWindow::readParamFromFile(std::string filein) {
         ui->FFTcheckBox->setChecked(par->getflagFFT());
         ui->ReducecheckBox->setChecked(par->getflagReduce());
     } 
-    if (par->getflagHanning()) {
+    if (par->getflagSmoothSpectral()) {
         setSmoothFlag(Qt::Checked);
         ui->HanninggroupBox->setChecked(true);
-        size_t hanning_window = par->getHanningWindow();
+        size_t hanning_window = par->getWindowSize();
         if (hanning_window%2==0) hanning_window+=1;
         ui->HanningspinBox->setValue(hanning_window);
     }
@@ -424,6 +431,7 @@ void BBaroloWindow::readParamFromFile(std::string filein) {
     }
 }
 
+
 void BBaroloWindow::writeParamFile(QString file) {
 
     using namespace std;
@@ -468,8 +476,8 @@ void BBaroloWindow::writeParamFile(QString file) {
             if (ui->SecondstagecheckBox->isChecked()) {
                 out << setw(n) << "TWOSTAGE" << "true" << endl;
                 if (ui->PolynspinBox->value()==-1)
-                    out << setw(n) << "POLYN" << "bezier" << endl;
-                else out << setw(n) << "POLYN" << ui->PolynspinBox->value() << endl;
+                    out << setw(n) << "REGTYPE" << "bezier" << endl;
+                else out << setw(n) << "REGTYPE" << ui->PolynspinBox->value() << endl;
             }
             else out << setw(n) << "TWOSTAGE" << "false" << endl;
             if (ui->ErrorsradioButton->isChecked())
@@ -555,10 +563,11 @@ void BBaroloWindow::writeParamFile(QString file) {
         else out << setw(n) << "SMOOTH" << "false" << endl;
         
         if (ui->HanninggroupBox->isChecked()) {
-            out << setw(n) << "HANNING" << "true" << endl;
-            out << setw(n) << "HANNING_SIZE" << ui->HanningspinBox->value();
+            out << setw(n) << "SMOOTHSPEC" << "true" << endl;
+            out << setw(n) << "WINDOW_TYPE" << "HANNING";
+            out << setw(n) << "WINDOW_SIZE" << ui->HanningspinBox->value();
         }
-        else out << setw(n) << "HANNING" << "false" << endl;
+        else out << setw(n) << "SMOOTHSPEC" << "false" << endl;
     }
 
 
@@ -593,6 +602,7 @@ void BBaroloWindow::writeParamFile(QString file) {
     out.close();
 }
 
+
 void BBaroloWindow::readModelParam(Param *p) {
 
     ui->FreeParametersframe->setEnabled(true);
@@ -808,6 +818,7 @@ void BBaroloWindow::readModelParam(Param *p) {
     if (par.flagADRIFT) ui->AdriftcheckBox->setChecked(true);
 }
 
+
 void BBaroloWindow::writeModelParam(std::ofstream &out) {
 
     int n=20;
@@ -927,6 +938,7 @@ void BBaroloWindow::writeModelParam(std::ofstream &out) {
     if (ui->AdriftcheckBox->isChecked()) out << setw(n) << "ADRIFT" << "true";
 }
 
+
 void BBaroloWindow::plotParameters(){
 
     const unsigned nsubplot = ui->SecondstagecheckBox->isChecked() ? 2 : 1;
@@ -934,7 +946,7 @@ void BBaroloWindow::plotParameters(){
 
     for (unsigned j=0; j<nsubplot; j++) {
         std::vector<std::vector<double> > allData;
-        std::string ringlog = out_path.toStdString()+"ringlog"+(QString::number(j+1)).toStdString()+".txt";
+        std::string ringlog = out_path.toStdString()+"rings_final"+(QString::number(j+1)).toStdString()+".txt";
         if (!getData(allData,ringlog,false)) continue;
 
         int nr = allData.size();
