@@ -62,7 +62,6 @@ void Detection<T>::defaultDetection() {
     xCentroid = 0.;
     yCentroid = 0.;
     zCentroid = 0.;
-    centreType="centroid";
     negSource = false; 
     flagText="";
     id = -1;
@@ -137,7 +136,6 @@ Detection<T>& Detection<T>::operator= (const Detection<T>& d) {
     this->xCentroid    = d.xCentroid;
     this->yCentroid    = d.yCentroid;
     this->zCentroid    = d.zCentroid;
-    this->centreType   = d.centreType;
     this->negSource    = d.negSource;
     this->flagText     = d.flagText;
     this->id           = d.id;
@@ -166,12 +164,6 @@ Detection<T>& Detection<T>::operator= (const Detection<T>& d) {
     this->w50          = d.w50;
     this->v50min       = d.v50min;
     this->v50max       = d.v50max;
-    this->posPrec      = d.posPrec;
-    this->xyzPrec      = d.xyzPrec;
-    this->fintPrec     = d.fintPrec;
-    this->fpeakPrec    = d.fpeakPrec;
-    this->velPrec      = d.velPrec;
-    this->snrPrec      = d.snrPrec;
     return *this;
 }
 
@@ -191,7 +183,7 @@ Detection<T> operator+ (Detection<T> &lhs, Detection<T> &rhs) {
 
 
 template <class T>  
-float Detection<T>::getXcentre() {
+float Detection<T>::getXcentre(std::string centreType) {
     
     if(centreType=="peak") return xpeak;
     else if(centreType=="average") return this->getXaverage();
@@ -200,7 +192,7 @@ float Detection<T>::getXcentre() {
 
 
 template <class T>  
-float Detection<T>::getYcentre() {
+float Detection<T>::getYcentre(std::string centreType) {
     
     if(centreType=="peak") return ypeak;
     else if(centreType=="average") return this->getYaverage();
@@ -209,13 +201,30 @@ float Detection<T>::getYcentre() {
 
 
 template <class T>
-float Detection<T>::getZcentre() {
+float Detection<T>::getZcentre(std::string centreType) {
     
     if(centreType=="peak") return zpeak;
     else if(centreType=="average") return this->getZaverage();
     else return zCentroid;
 }
 
+template <class T>
+float Detection<T>::getVsys() {
+
+    float vsys;
+    if (w20>1.5*w50) {
+        // If W20 and W50 differ a lot, there is something weird. Just use W20
+        vsys = (v20max+v20min)/2.;
+    }
+    else {
+        float Vsys_20 = (v20max+v20min)/2.;
+        float Vsys_50 = (v50max+v50min)/2.;
+        vsys = (Vsys_20+Vsys_50)/2.;
+    }
+
+    return vsys;
+
+}
 
 template <class T>
 void Detection<T>::setOffsets(long Xoffset, long Yoffset, long Zoffset) {
@@ -285,7 +294,7 @@ bool Detection<T>::hasEnoughChannels(int minNumber) {
 
 
 template <class T>
-bool Detection<T>::canMerge(Detection<T> &other, Param &par) {
+bool Detection<T>::canMerge(Detection<T> &other, SEARCH_PAR &par) {
     
     bool near = isNear(other, par);
     if(near) return isClose(other, par);
@@ -294,11 +303,11 @@ bool Detection<T>::canMerge(Detection<T> &other, Param &par) {
 
 
 template <class T>
-bool Detection<T>::isNear(Detection<T> &other, Param &par) {
+bool Detection<T>::isNear(Detection<T> &other, SEARCH_PAR &par) {
 
-    bool flagAdj = par.getParSE().flagAdjacent;
-    float threshS = par.getParSE().threshSpatial;
-    float threshV = par.getParSE().threshVelocity;
+    bool flagAdj = par.flagAdjacent;
+    float threshS = par.threshSpatial;
+    float threshV = par.threshVelocity;
     
     long gap;
     if(flagAdj) gap = 1;
@@ -327,13 +336,13 @@ bool Detection<T>::isNear(Detection<T> &other, Param &par) {
 
 
 template <class T>
-bool Detection<T>::isClose(Detection<T> &other, Param &par)  {
+bool Detection<T>::isClose(Detection<T> &other, SEARCH_PAR &par)  {
    
     bool close = false;   
     
-    bool flagAdj = par.getParSE().flagAdjacent;
-    float threshS = par.getParSE().threshSpatial;
-    float threshV = par.getParSE().threshVelocity;
+    bool flagAdj = par.flagAdjacent;
+    float threshS = par.threshSpatial;
+    float threshV = par.threshVelocity;
     // 
     // If we get to here, the pixel ranges overlap -- so we do a
     // pixel-by-pixel comparison to make sure they are actually
@@ -530,29 +539,9 @@ void Detection<T>::calcWCSparams(Header &head) {
  
     haveParams = true;
 
-    std::string cunit = head.Cunit(0);  
-    double arcsconv, degconv;
-    if (cunit=="DEGREE" || cunit=="DEGREES" || cunit=="DEG" ||
-        cunit=="degree" || cunit=="degrees" || cunit=="deg") {
-        degconv  = 1;
-        arcsconv = 3600;
-    }
-    else if (cunit=="ARCSEC" || cunit=="ARCS" ||
-             cunit=="arcsec" || cunit=="arcs") {
-        degconv  = 1/3600.;
-        arcsconv = 1;
-    }
-    else if (cunit=="ARCMIN" || cunit=="ARCM" ||
-             cunit=="arcmin" || cunit=="arcm") {
-        degconv  = 1/60.;
-        arcsconv = 60;
-    }
-    else {
-        std::cout << "Error (unknown CUNIT for RA-DEC): ";
-        std::cout << "cannot convert to ARCSEC.\n";
-        std::cout << cunit;
-        std::terminate();
-    }
+    std::string cunit = head.Cunit(1);
+    double arcconv = arcsconv(cunit);
+    double degconv = arcconv/3600.;
     
     lngtype = head.Ctype(0);
     lattype = head.Ctype(1);
@@ -573,12 +562,12 @@ void Detection<T>::calcWCSparams(Header &head) {
     raMax *= degconv;
     deMin *= degconv;
     deMax *= degconv;
-    raWidth   = angularSeparation(raMin,dec,raMax,dec)*arcsconv;
-    decWidth  = angularSeparation(ra,deMin,ra,deMax)*arcsconv;
+    raWidth   = angularSeparation(raMin,dec,raMax,dec)*arcconv;
+    decWidth  = angularSeparation(ra,deMin,ra,deMax)*arcconv;
 
     Object2D<T> spatMap = this->getSpatialMap();
     std::pair<double,double> axes = spatMap.getPrincipleAxes();
-    float PixScale = ((fabs(head.Cdelt(0))+fabs(head.Cdelt(1)))/2.)*arcsconv;
+    float PixScale = ((fabs(head.Cdelt(0))+fabs(head.Cdelt(1)))/2.)*arcconv;
     majorAxis = std::max(axes.first,axes.second)*PixScale;
     minorAxis = std::min(axes.first,axes.second)*PixScale;
     posang = spatMap.getPositionAngle()*180./M_PI;
@@ -592,7 +581,6 @@ void Detection<T>::calcWCSparams(Header &head) {
     velWidth = fabs(velMax-velMin);
 
     flagWCS = true;
-    
 
 }
   //--------------------------------------------------------------------
@@ -869,11 +857,6 @@ void Detection<T>::calcVelWidths(long zdim, T *intSpec, Header &head) {
 
     w20 = fabs(v20min-v20max);
     w50 = fabs(v50min-v50max);
-    float Vsys_20 = (v20max+v20min)/2.;
-    float Vsys_50 = (v50max+v50min)/2.;
-    vsys = (Vsys_20+Vsys_50)/2.;
-
-
 }
   //--------------------------------------------------------------------
 
@@ -893,30 +876,22 @@ void Detection<T>::calcVelWidths(T *fluxArray, long *dim, Header &head) {
     ///  \param head FitsHeader object that contains the WCS information.
 
     if(dim[2]>2){
-
         float *intSpec = new float[dim[2]];
         long size=dim[0]*dim[1]*dim[2];
         std::vector<bool> mask(size,true); 
         getIntSpec(*this,fluxArray,dim,mask,1.,intSpec);
-    
         calcVelWidths(dim[2],intSpec,head);
-
         delete [] intSpec;
-
     }
     else{
         v50min = v20min = velMin;
         v50max = v20max = velMax;
         w20 = fabs(v20min-v20max);
         w50 = fabs(v50min-v50max);
-        float Vsys_20 = (v20max+v20min)/2.;
-        float Vsys_50 = (v50max+v50min)/2.;
-        vsys = (Vsys_20+Vsys_50)/2.;
     }
-
-
-
 }
+
+
   //--------------------------------------------------------------------
 
 
