@@ -156,8 +156,6 @@ void Galmod<T>::defaults() {
     relint.push_back(1);
            
 }
-template void Galmod<float>::defaults();
-template void Galmod<double>::defaults();
 
 
 template <class T>
@@ -165,8 +163,6 @@ Galmod<T>::Galmod() {
     
     defaults();
 }
-template Galmod<float>::Galmod();
-template Galmod<double>::Galmod();
 
 
 template <class T>
@@ -176,8 +172,6 @@ Galmod<T>::~Galmod() {
     if (outDefined) delete out;
     if (ringDefined) delete r;
 }
-template Galmod<float>::~Galmod();
-template Galmod<double>::~Galmod();
 
 
 template <class T>
@@ -185,8 +179,6 @@ Galmod<T>::Galmod(const Galmod<T> &g) {
 
     operator=(g);
 }
-template Galmod<float>::Galmod(const Galmod<float>&);
-template Galmod<double>::Galmod(const Galmod<double>&);
 
 
 template <class T>
@@ -247,8 +239,6 @@ Galmod<T>& Galmod<T>::operator=(const Galmod &g) {
     
     return *this;
 }
-template Galmod<float>& Galmod<float>::operator=(const Galmod<float>&);
-template Galmod<double>& Galmod<double>::operator=(const Galmod<double>&);
 
 
 template <class T>
@@ -257,11 +247,11 @@ void Galmod<T>::input(Cube<T> *c, int *Boxup, int *Boxlow, Rings<T> *rings,
     
     /// This function sets all parameters needed to use the Galmod Object. A description
     /// of input is given in the Galmod class definition file (galmod.hh). 
-        
+
     initialize(c, Boxup, Boxlow);
-    
+
     ringIO(rings);
-    
+
     setOptions(LTYPE, CMODE, CDENS, ISEED);
     
     int nvtmp = NV;
@@ -276,8 +266,6 @@ void Galmod<T>::input(Cube<T> *c, int *Boxup, int *Boxlow, Rings<T> *rings,
     readytomod=true;
     
 }
-template void Galmod<float>::input(Cube<float>*,int*,int*,Rings<float>*,int,int,int,float,int);
-template void Galmod<double>::input(Cube<double>*,int*,int*,Rings<double>*,int,int,int,float,int);
 
 
 template <class T>
@@ -290,8 +278,6 @@ void Galmod<T>::input(Cube<T> *c, Rings<T> *rings, int NV, int LTYPE, int CMODE,
     
     input(c,Bup,Blo,rings,NV,LTYPE,CMODE,CDENS,ISEED);
 }
-template void Galmod<float>::input(Cube<float>*,Rings<float>*,int,int,int,float,int);
-template void Galmod<double>::input(Cube<double>*,Rings<double>*,int,int,int,float,int);
 
 
 template <class T>
@@ -322,8 +308,6 @@ void Galmod<T>::setOptions(int LTYPE, int CMODE, float CDENS, int ISEED) {
     gaussia   = std::normal_distribution<float>(0.,1.);
         
 }
-template void Galmod<float>::setOptions(int,int,float,int);
-template void Galmod<double>::setOptions(int,int,float,int);
 
 
 template <class T>
@@ -340,8 +324,6 @@ bool Galmod<T>::calculate() {
     }
     return true;
 }
-template bool Galmod<float>::calculate();
-template bool Galmod<double>::calculate();
 
 
 template <class T>
@@ -355,11 +337,11 @@ bool Galmod<T>::smooth(bool usescalefac) {
     
     //Beam oldbeam = {pixsize[0]*60, pixsize[1]*60, 0};
     Beam oldbeam = {0., 0., 0};    
-    Beam newbeam = {in->Head().Bmaj()*arcmconv*60,
-                    in->Head().Bmin()*arcmconv*60,
+    Beam newbeam = {in->Head().Bmaj()*3600.,
+                    in->Head().Bmin()*3600.,
                     in->Head().Bpa()};
-    
-    Smooth3D<T> *smoothed = new Smooth3D<T>;    
+
+    Smooth3D<T> *smoothed = new Smooth3D<T>;
     smoothed->setUseScalefac(usescalefac);
     smoothed->setUseBlanks(false);
     if(!smoothed->smooth(out, oldbeam, newbeam, out->Array(), out->Array()))
@@ -372,8 +354,6 @@ bool Galmod<T>::smooth(bool usescalefac) {
     
     return true;
 }
-template bool Galmod<float>::smooth(bool);
-template bool Galmod<double>::smooth(bool);
 
 
 template <class T>
@@ -425,8 +405,6 @@ void Galmod<T>::normalize() {
     }
     
 }
-template void Galmod<float>::normalize();
-template void Galmod<double>::normalize();
 
 
 template <class T>
@@ -449,17 +427,17 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
 
         if (cunit[i]=="DEGREE" || cunit[i]=="DEGREES" || cunit[i]=="DEG" ||
             cunit[i]=="degree" || cunit[i]=="degrees" || cunit[i]=="deg") 
-                arcmconv = 60;
+                arcmconv = 60.;
         else if (cunit[i]=="ARCSEC" || cunit[i]=="ARCS" ||
-                 cunit[i]=="arcsec" || cunit[i]=="arcs") 
-                arcmconv = 1/60;
+                 cunit[i]=="arcsec" || cunit[i]=="arcs")
+                arcmconv = 1/60.;
         else if (cunit[i]=="ARCMIN" || cunit[i]=="ARCM" ||
                  cunit[i]=="arcmin" || cunit[i]=="arcm") 
-                arcmconv = 1;
+                arcmconv = 1.;
         else {
-            std::cout << "GALMOD error (unknown CUNIT for RA-DEC): ";
-            std::cout << "cannot convert to ARCMIN.\n";
-            std::cout << cunit[i];
+            std::cerr << "GALMOD error (unknown CUNIT for RA-DEC): ";
+            std::cerr << "cannot convert to ARCMIN.\n";
+            std::cerr << cunit[i];
             std::terminate(); 
         }
         cdelt[i]   = cdelt[i]*arcmconv;
@@ -467,7 +445,7 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         bsize[i]   = bhi[i]-blo[i]; 
     }
     pixarea=pixsize[0]*pixsize[1];
-    
+
     crota2=in->Head().Crota();
     crota2=crota2*M_PI/180.;
     
@@ -544,8 +522,8 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         else if (cunit3=="mhz") hzconv = 1.0E06;
         else if (cunit3=="ghz") hzconv = 1.0E09;
         else {
-            std::cout << "GALMOD error (unknown CUNIT3): cannot convert to Hz.\n";
-            std::cout << cunit3;
+            std::cerr << "GALMOD error (unknown CUNIT3): cannot convert to Hz.\n";
+            std::cerr << cunit3;
             std::terminate(); 
         }
         
@@ -553,11 +531,11 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         cdelt3=cdelt3*hzconv;  
         
         /// Information about frequency/velocity axis and conversions.
-        freq0 = c->Head().Freq0()/(1+c->Head().Redshift());      
+        freq0 = c->Head().Freq0()/(1+c->Head().Redshift());
         if (freq0==0) {
             freq0 = 0.1420405751786E10;
-            std::cout << "Header item FREQ0 not found. Assuming " << freq0;
-            std::cout << std::endl; 
+            std::cerr << "Header item FREQ0 not found. Assuming " << freq0;
+            std::cerr << std::endl;
         }
         
         double crvalfreq = c->Head().Crval(2)*hzconv;
@@ -582,8 +560,8 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         else if (cunit3=="km/s") msconv = 1.0E03;
         else if ("cm/s") msconv = 1.0E-03;
         else {
-            std::cout << "GALMOD error (unknown CUNIT3): cannot convert to M/S.\n";
-            std::cout << cunit3;
+            std::cerr << "GALMOD error (unknown CUNIT3): cannot convert to M/S.\n";
+            std::cerr << cunit3;
             std::terminate(); 
         }
         
@@ -591,8 +569,8 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         freq0 = c->Head().Freq0()/(1+c->Head().Redshift());      
         if (freq0==0) {
             freq0 = 0.1420405751786E10;
-            std::cout << "Header item FREQ0 not found. Assuming " << freq0;
-            std::cout << std::endl; 
+            std::cerr << "Header item FREQ0 not found. Assuming " << freq0;
+            std::cerr << std::endl;
         }
         
         crval3=crval3*msconv;
@@ -602,7 +580,7 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
         
     }
     else { 
-        std::cout << "Unknown axis type: no velocities along spectral axis.\n";
+        std::cerr << "Unknown axis type: no velocities along spectral axis.\n";
         std::terminate();
     }
     
@@ -615,8 +593,6 @@ void Galmod<T>::initialize(Cube<T> *c, int *Boxup, int *Boxlow) {
     sig_instr = nch*chwidth;
     
 }
-template void Galmod<float>::initialize(Cube<float>*,int*,int*);
-template void Galmod<double>::initialize(Cube<double>*,int*,int*);
 
 
 template <class T>
@@ -653,11 +629,11 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
         uradii[i]=rings->radii[i]/60.;
         if (uradii[i-1]+r->radsep>=uradii[i]) {
             if (uradii[i-1]>uradii[i]) {
-                std::cout << "GALMOD error: Radii not in increasing order.\n";
+                std::cerr << "GALMOD error: Radii not in increasing order.\n";
                 std::terminate();
             }
             else {
-                std::cout << "GALMOD error: Radius separation too small.\n";
+                std::cerr << "GALMOD error: Radius separation too small.\n";
                 std::terminate();
             }
         }
@@ -681,20 +657,20 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
         
         uvdisp[i] = rings->vdisp[i]*1000;
         if (uvdisp[i]<0) {
-            std::cout << "GALMOD error: Negative velocity dispersion not allowed.\n";
+            std::cerr << "GALMOD error: Negative velocity dispersion not allowed.\n";
             std::terminate();
         }
         
         udens[i]=rings->dens[i]/1.0E20;
 
         if (udens[i]<0) {
-            std::cout << "GALMOD error: Negative column-density not allowed.\n";
+            std::cerr << "GALMOD error: Negative column-density not allowed.\n";
             std::terminate(); 
         }
         
         uz0[i]=rings->z0[i]/60.;
         if (uz0[i]<0) {
-            std::cout << "GALMOD error: Negative scale height not allowed.\n";
+            std::cerr << "GALMOD error: Negative scale height not allowed.\n";
             std::terminate(); 
         }
         
@@ -706,7 +682,7 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
         uypos[i]  = rings->ypos[i]+1;
         uvsys[i]  = rings->vsys[i]*1000;
     }
-    
+
     if (uradii[0]!=0 && empty) r->radii.push_back(uradii[0]+r->radsep/2.0);
     else {
         r->radii.push_back(r->radsep/2.0);
@@ -727,7 +703,7 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
             r->radii.push_back(r->radii.back()+r->radsep);
         }
     }
-    
+
     for (int i=1; i<nur; i++) {
         T dur       = uradii[i]-uradii[i-1];
         T dvrotdr   = (uvrot[i]-uvrot[i-1])/dur;
@@ -762,7 +738,7 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
             r->radii.push_back(r->radii.back()+r->radsep);
         }
     }
-    
+
     r->radii.pop_back();
     r->nr=r->radii.size();      
     
@@ -783,21 +759,19 @@ void Galmod<T>::ringIO(Rings<T> *rings) {
     delete [] uypos;
     delete [] uvsys;
 }
-template void Galmod<float>::ringIO(Rings<float>*);
-template void Galmod<double>::ringIO(Rings<double>*);
 
-///*
+/*
 template <class T>
 void Galmod<T>::galmod() {
 
+    std::cout << "GALMOD PROTO" << std::endl;
+
     const double twopi = 2*M_PI;
     const int buflen=bsize[0]*bsize[1]*nsubs+1;
     T *datbuf = new T [buflen];
             
-    bool verb = in->pars().isVerbose();
-    ProgressBar bar(" Modeling... ",false);;
-    bar.setShowbar(in->pars().getShowbar());
-    if (verb) bar.init(r->nr);
+    ProgressBar bar(false,in->pars().isVerbose(),in->pars().getShowbar());
+    bar.init(" Modeling... ",r->nr);
     
     int isd = iseed;
 //  Get number of velocity profiles that will be done.
@@ -809,7 +783,8 @@ void Galmod<T>::galmod() {
     
     // ==>> Loop over standard rings.
     for (int ir=0; ir<r->nr; ir++) {
-        if (verb) bar.update(ir+1);
+        bar.update(ir+1);
+        if (r->dens[ir]==0) continue;
 //      Get radius
         double rtmp = r->radii[ir];
 //      Get number of clouds inside ring.
@@ -821,6 +796,143 @@ void Galmod<T>::galmod() {
             continue;
         }
 //      Get values of ir-radius.            
+        double vrottmp = r->vrot[ir];
+        double vradtmp = r->vrad[ir];
+        double vverttmp= r->vvert[ir];
+        //  The VDISP should be such that VDISP^2 + chwidth^2 / 12 = sig_instr^2 + sig_v^2
+        //double vdisptmp= sqrt(r->vdisp[ir]*r->vdisp[ir]+sig_instr*sig_instr-(chwidth*chwidth)/12.);
+        double vdisptmp= sqrt(r->vdisp[ir]*r->vdisp[ir]+sig_instr*sig_instr);
+        double vsystmp = r->vsys[ir];
+        double z0tmp   = 0;//r->z0[ir];
+        double dvdztmp = r->dvdz[ir];
+        double zcyltmp = r->zcyl[ir];
+        double sinc    = sin(r->inc[ir]);
+        double cinc    = cos(r->inc[ir]);
+        double spa     = sin(r->phi[ir])*cos(crota2)+cos(r->phi[ir])*sin(crota2);
+        double cpa     = cos(r->phi[ir])*cos(crota2)-sin(r->phi[ir])*sin(crota2);
+        double nvtmp   = nv[ir];
+        float  fluxsc  = r->dens[ir]*twopi*rtmp*r->radsep/(nc*nvtmp);
+
+
+//        double zs = 0.14*rtmp;
+        double zs = 2*pow(rtmp/20,0.5);
+
+        double zstart[2] = {r->z0[ir],-r->z0[ir]};
+        //double zstart[2] = {zs,-zs};
+
+        //std::cout << rtmp << " "  << zstart[0] << "  " << zstart[1] << std::endl;
+
+        for (int kk=0; kk<2; kk++) {
+            if (kk==1) fluxsc /= 2;
+
+            // ==>> Loop over clouds inside each ring.
+            for (int ic=0; ic<nc; ic++) {
+                //          Get radius inside ring. The range includes the inner boundary,
+                //          excludes the outer boundary. The probability of a radius inside
+                //          a ring is proportional to the total radius and thus the
+                //          surface density of the clouds is constant over the area of the ring.
+                double ddum = fabs(fran());                        // STD library
+                //double ddum = double(iran(isd))/double(nran);       // Classic galmod
+                double R    = sqrt(pow((rtmp-0.5*r->radsep),2)+2*r->radsep*rtmp*ddum);
+                //          Get azimuth and its sine and cosine.
+                double az   = twopi*fabs(fran());                 // STD library
+                //double az   = twopi*double(iran(isd))/double(nran); // Classic galmod
+                double saz  = sin(az);
+                double caz  = cos(az);
+                //          Get height above the plane of the ring using a random deviate
+                //          drawn from density profile of the layer.
+                double z    = zstart[kk]+fdev(isd)*z0tmp;
+                //          Get position in the plane of the sky with respect to the major
+                //          and minor axes of the spiral galaxy.
+                double x    = R*caz;
+                double y    = R*saz*cinc-z*sinc;
+                //          Get grid of this position, check if it is inside area of box.
+                long grid[2] = {lround(r->xpos[ir]+(x*spa-y*cpa)/cdelt[0]),
+                                lround(r->ypos[ir]+(x*cpa+y*spa)/cdelt[1])};
+                if (grid[0]<=blo[0] || grid[0]>bhi[0]) continue;
+                if (grid[1]<=blo[1] || grid[1]>bhi[1]) continue;
+
+                //
+                //          Get profile number of current pixel and check if position is in
+                //          range of positions of profiles that are currently being done.
+                //          If outside any of the ranges, jump to next cloud.
+                int iprof = (grid[1]-blo[1])*bsize[0]+grid[0]-blo[0]-bsize[0];
+                //          Get systematic velocity of cloud.
+                double vsys = vsystmp+(vrottmp*caz+vradtmp*saz)*sinc;
+                //          Adding vertical rotational gradient after zcyl
+                if (abs(z)>zcyltmp) vsys = vsystmp+((vrottmp-dvdztmp*(abs(z)-zcyltmp))*caz+vradtmp*saz)*sinc;
+                //          Adding vertical velocity
+                if (z>0.) vsys += vverttmp*cinc;
+                else vsys -= vverttmp*cinc;     // <--- The sign here depends on the meaning of vvert
+                
+
+                //          MODIFIED BUILDING PROFILE FOR MULTIPLE LINES
+                for (int iv=0; iv<nvtmp; iv++) {
+                    double vdev = gaussia(generator)*vdisptmp;        // STD library
+                    //double vdev = gasdev(isd)*vdisptmp;                 // Classic galmod
+                    for (int nl=0; nl<nlines; nl++) {
+                        double v     = vsys+vdev+relvel[nl];
+                        int isubs = lround(velgrid(v)+crpix3-1);
+                        if (isubs<0 || isubs>=nsubs) continue;
+                        int idat  = iprof+isubs*nprof;
+                        datbuf[idat] = datbuf[idat]+relint[nl]*fluxsc*cd2i[isubs];
+                    }
+                }
+            }
+        }
+    }
+    
+//  Write data to output Cube.      
+    for (int isubs=0; isubs<nsubs; isubs++) {
+        int pixstart=isubs*bsize[0]*bsize[1];
+        int idat=1+isubs*nprof;
+        for (int i=0; i<nprof; i++)
+            out->Array(pixstart+i)=datbuf[idat+i];
+    }
+    
+    bar.fillSpace("OK.\n");
+    
+    delete [] datbuf;
+
+    modCalculated=true;
+
+}
+*/
+
+///*
+template <class T>
+void Galmod<T>::galmod() {
+
+    const double twopi = 2*M_PI;
+    const int buflen=bsize[0]*bsize[1]*nsubs+1;
+    T *datbuf = new T [buflen];
+
+    ProgressBar bar(false,in->pars().isVerbose(),in->pars().getShowbar());
+    bar.init(" Modeling... ",r->nr);
+
+    int isd = iseed;
+//  Get number of velocity profiles that will be done.
+    int nprof = bsize[0]*bsize[1];
+//  Initialize data buffer on zero.
+    for (int i=0; i<buflen; i++) datbuf[i]=0.0;
+//  Convenient random generator functions
+    auto fran = std::bind(uniform, generator);
+
+    // ==>> Loop over standard rings.
+    for (int ir=0; ir<r->nr; ir++) {
+        bar.update(ir+1);
+        if (r->dens[ir]==0) continue;
+//      Get radius
+        double rtmp = r->radii[ir];
+//      Get number of clouds inside ring.
+        int nc = lround(cdens*pow(r->dens[ir],cmode)*twopi*rtmp*r->radsep/pixarea);
+        if (nc==0) {
+            std::cerr << " GALMOD ERROR: No clouds used. Choose higher CDENS " << std::endl;
+            std::terminate();
+//          Do next ring, jump to end of loop for rings.
+            continue;
+        }
+//      Get values of ir-radius.
         double vrottmp = r->vrot[ir];
         double vradtmp = r->vrad[ir];
         double vverttmp= r->vvert[ir];
@@ -837,12 +949,12 @@ void Galmod<T>::galmod() {
         double cpa     = cos(r->phi[ir])*cos(crota2)-sin(r->phi[ir])*sin(crota2);
         double nvtmp   = nv[ir];
         float  fluxsc  = r->dens[ir]*twopi*rtmp*r->radsep/(nc*nvtmp);
-           
+
 // ==>> Loop over clouds inside each ring.
         for (int ic=0; ic<nc; ic++) {
 //          Get radius inside ring. The range includes the inner boundary,
 //          excludes the outer boundary. The probability of a radius inside
-//          a ring is proportional to the total radius and thus the 
+//          a ring is proportional to the total radius and thus the
 //          surface density of the clouds is constant over the area of the ring.
             double ddum = fabs(fran());                        // STD library
             //double ddum = double(iran(isd))/double(nran);       // Classic galmod
@@ -850,18 +962,18 @@ void Galmod<T>::galmod() {
 //          Get azimuth and its sine and cosine.
             double az   = twopi*fabs(fran());                 // STD library
             //double az   = twopi*double(iran(isd))/double(nran); // Classic galmod
-            double saz  = sin(az); 
-            double caz  = cos(az); 
+            double saz  = sin(az);
+            double caz  = cos(az);
 //          Get height above the plane of the ring using a random deviate
 //          drawn from density profile of the layer.
             double z    = fdev(isd)*z0tmp;
 //          Get position in the plane of the sky with respect to the major
 //          and minor axes of the spiral galaxy.
             double x    = R*caz;
-            double y    = R*saz*cinc-z*sinc;                                                 
+            double y    = R*saz*cinc-z*sinc;
 //          Get grid of this position, check if it is inside area of box.
             long grid[2] = {lround(r->xpos[ir]+(x*spa-y*cpa)/cdelt[0]),
-                            lround(r->ypos[ir]+(x*cpa+y*spa)/cdelt[1])};             
+                            lround(r->ypos[ir]+(x*cpa+y*spa)/cdelt[1])};
             if (grid[0]<=blo[0] || grid[0]>bhi[0]) continue;
             if (grid[1]<=blo[1] || grid[1]>bhi[1]) continue;
 
@@ -870,29 +982,29 @@ void Galmod<T>::galmod() {
 //            float xr = (-(grid[0]-r->pos[0])*spa+(grid[1]-r->pos[1])*cpa);
 //            float yr = (-(grid[0]-r->pos[0])*cpa-(grid[1]-r->pos[1])*spa)/cinc;
 //            rr = sqrt(xr*xr+yr*yr);
-//            if (rr<0.1) theta = 0.0;            
-//            else theta = atan2(yr, xr)/M_PI*180;    
-//                
+//            if (rr<0.1) theta = 0.0;
+//            else theta = atan2(yr, xr)/M_PI*180;
+//
 //            int side = 1;
 //            bool use;
-//            switch (side) {                     // Which side of galaxy.                    
-//                case 1:                         //< Receding half.                              
-//                    use = (fabs(theta)<=90.0);      
+//            switch (side) {                     // Which side of galaxy.
+//                case 1:                         //< Receding half.
+//                    use = (fabs(theta)<=90.0);
 //                    break;
-//                case 2:                         //< Approaching half. 
+//                case 2:                         //< Approaching half.
 //                    use = (fabs(theta)>=90.0);
 //                    break;
 //                case 3:                         //< Both halves.
 //                    use = 1;
 //                    break;
-//                default: 
-//                    break;  
+//                default:
+//                    break;
 //            }
-//                
+//
 //            if (!use) continue;
-                  
+
 //          //////////// >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-//              
+//
 //          Get profile number of current pixel and check if position is in
 //          range of positions of profiles that are currently being done.
 //          If outside any of the ranges, jump to next cloud.
@@ -903,9 +1015,9 @@ void Galmod<T>::galmod() {
             if (abs(z)>zcyltmp) vsys = vsystmp+((vrottmp-dvdztmp*(abs(z)-zcyltmp))*caz+vradtmp*saz)*sinc;
 //          Adding vertical velocity
             if (z>0.) vsys += vverttmp*cinc;
-            else vsys += vverttmp*cinc;     //// <--- Check this. In galmod is a +
-                
-                    
+            else vsys -= vverttmp*cinc;     // <--- The sign here depends on the meaning of vvert
+
+
 //          ORIGINAL GALMOD BUILDING PROFILES
 // ==>>     Build velocity profile.
 //            for (int iv=0; iv<nvtmp; iv++) {
@@ -915,10 +1027,10 @@ void Galmod<T>::galmod() {
 //              Get grid of velocity along FREQ-OHEL or VELO axis.
 //              If a grid is not in the range, jump to next velocity profile.
 //                int isubs = lround(velgrid(v)+crpix3-1);
-//                if (isubs<0 || isubs>=nsubs) continue;                  
+//                if (isubs<0 || isubs>=nsubs) continue;
 //               int idat  = iprof+isubs*nprof;
 //              Convert HI atom flux per pixel to flux per pixel of 21cm
-//              radiation expressed in W.U. and add subcloud to the data
+//              radiation expressed in Jy and add subcloud to the data
 //              buffer.
 //                datbuf[idat] = datbuf[idat]+fluxsc*cd2i[isubs];
 //            }
@@ -937,25 +1049,23 @@ void Galmod<T>::galmod() {
             }
         }
     }
-    
-//  Write data to output Cube.      
+
+//  Write data to output Cube.
     for (int isubs=0; isubs<nsubs; isubs++) {
         int pixstart=isubs*bsize[0]*bsize[1];
         int idat=1+isubs*nprof;
         for (int i=0; i<nprof; i++)
             out->Array(pixstart+i)=datbuf[idat+i];
     }
-    
-    if (verb) bar.fillSpace("OK.\n");
-    
+
+    bar.fillSpace("OK.\n");
+
     delete [] datbuf;
 
     modCalculated=true;
 
 }
-template void Galmod<float>::galmod();
-template void Galmod<double>::galmod();
-//*/  
+//*/
 
 /*
 // GALMOD FOR SMC
@@ -1178,8 +1288,6 @@ void Galmod<T>::galmod() {
     modCalculated=true;
     
 }
-template void Galmod<float>::galmod();
-template void Galmod<double>::galmod();
 */
 
 
@@ -1211,9 +1319,9 @@ void Galmod<T>::NHItoRAD(){
 /// The column density of the HI is in units of 1E20 atoms per cm2.
 /// Velocities are in M/S and frequencies in HZ. Solid angle is in steradians.
 /// Intensity is flux per one steradian solid angle. It is defined
-/// as the flux per beam of one square arcminute in W.U..
+/// as the flux per beam of one square arcminute in Jy.
 
-    double const wu = 5E-29;             // Conversion factor to W.U.
+    double const jy = 1E-26;             // Conversion factor to Jy
     double const K  = 1.3806488E-23;     // Boltzmann constant in Joule/Kelvin 
     
     for (int isubs=0; isubs<nsubs; isubs++) {
@@ -1235,17 +1343,15 @@ void Galmod<T>::NHItoRAD(){
             fac = 1.823E-05;       // This is 1.823E13 / 1E20 * 1E02
         } 
 
-        double ddum = 2*(K/wu)/(fac*(labsubs*labsubs))/fabs(cdelt3);
+        double ddum = 2*(K/jy)/(fac*(labsubs*labsubs))/fabs(cdelt3);
         // Following line removes dependency from lambda
-        //double ddum=2*(K/wu)/fac/fabs(cdelt3);
+        //double ddum=2*(K/jy)/fac/fabs(cdelt3);
         // The solid angle in the definition of the intensity is 
         // in steradians, thus convert it to square arcminutes.
-        cd2i[isubs]=ddum*pow((M_PI/(180.0*60.0)),2);
+        cd2i[isubs]= ddum*pow((M_PI/(180.0*60.0)),2);
     }
 
 }
-template void Galmod<float>::NHItoRAD();
-template void Galmod<double>::NHItoRAD();
 
 
 template <class T>  
@@ -1275,8 +1381,6 @@ double Galmod<T>::velgrid(double v) {
     return velg;
 
 }
-template double Galmod<float>::velgrid(double);
-template double Galmod<double>::velgrid(double);
 
 
 template <class T>
@@ -1308,8 +1412,7 @@ double Galmod<T>::fdev(int &idum){
 
     return Fdev;
 }
-template double Galmod<float>::fdev(int&);
-template double Galmod<double>::fdev(int&);
+
 
 //////////////////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -1338,8 +1441,6 @@ void Galmod_wind<T>::input(Cube<T> *c, int *Boxup, int *Boxlow, Shells<T> *shell
     this->readytomod=true;
     
 }
-template void Galmod_wind<float>::input(Cube<float>*,int*,int*,Shells<float>*,int,int,int,float,int);
-template void Galmod_wind<double>::input(Cube<double>*,int*,int*,Shells<double>*,int,int,int,float,int);
 
 
 template <class T>
@@ -1472,8 +1573,6 @@ void Galmod_wind<T>::shellIO(Shells<T> *shells) {
     delete [] uvsys;
     delete [] uvoan;
 }
-template void Galmod_wind<float>::shellIO(Shells<float>*);
-template void Galmod_wind<double>::shellIO(Shells<double>*);
 
 
 template <class T>
@@ -1488,9 +1587,6 @@ bool Galmod_wind<T>::calculate() {
     }
     return true;
 }
-template bool Galmod_wind<float>::calculate();
-template bool Galmod_wind<double>::calculate();
-
 
 
 template <class T>
@@ -1507,20 +1603,18 @@ void Galmod_wind<T>::galmod_wind() {
     int nprof = this->bsize[0]*this->bsize[1];
 //  Convenient random generator functions
     auto fran = std::bind(this->uniform, this->generator);
-    bool verb = this->in->pars().isVerbose();
 
-    ProgressBar bar(" Modeling outflow... ",false);
-    bar.setShowbar(this->in->pars().getShowbar());
+    ProgressBar bar(false,this->in->pars().isVerbose(),this->in->pars().getShowbar());
 
     int nthreads = this->in->pars().getThreads();
 #pragma omp parallel num_threads(nthreads)
 {
-    if (verb) bar.init(s->ns);
+    bar.init(" Modeling outflow... ",s->ns);
     
 #pragma omp for schedule (dynamic)
     // ==>> Loop over standard spherical shells.
     for (int ir=0; ir<s->ns; ir++) {
-        if (verb) bar.update(ir+1);
+        bar.update(ir+1);
 //      Get radius
         double rtmp = s->radii[ir];
 //      Get number of clouds inside ring.
@@ -1529,7 +1623,7 @@ void Galmod_wind<T>::galmod_wind() {
             std::cerr << " GALMOD ERROR: No clouds used. Choose higher CDENS " << std::endl;
             std::terminate();
         }
-//      Get values of ir-radius.            
+//      Get values of ir-radius.
         double vrottmp = s->vrot[ir];
         double vsphtmp = s->vsph[ir];
         double opentmp = s->openang[ir]/2.;
@@ -1599,14 +1693,19 @@ void Galmod_wind<T>::galmod_wind() {
             this->out->Array(pixstart+i)=datbuf[idat+i];
     }
     
-    if (verb) bar.fillSpace("OK.\n");
+    bar.fillSpace("OK.\n");
     delete [] datbuf;
     
     this->modCalculated=true;
 
 }
-template void Galmod_wind<float>::galmod_wind();
-template void Galmod_wind<double>::galmod_wind();
+
+
+// Explicit instantiation of the classes
+template class Galmod<float>;
+template class Galmod<double>;
+template class Galmod_wind<float>;
+template class Galmod_wind<double>;
 
 }
 
