@@ -55,8 +55,6 @@ GalWind<T>::GalWind(Cube<T> *c, T x0, T y0, T pa, T inc, T disp, T dens, T vsys,
 
     in->checkBeam();
 }
-template GalWind<float>::GalWind(Cube<float>*,float,float,float,float,float,float,float,float,float,float,int,int,int,int,int);
-template GalWind<double>::GalWind(Cube<double>*,double,double,double,double,double,double,double,double,double,double,int,int,int,int,int);
 
 
 template <class T>
@@ -72,8 +70,6 @@ GalWind<T>& GalWind<T>::operator= (const GalWind<T>& gw) {
     
     return *this;
 }
-template GalWind<float>& GalWind<float>::operator=(const GalWind<float>&);
-template GalWind<double>& GalWind<double>::operator=(const GalWind<double>&);
 
 
 template <class T>
@@ -90,8 +86,6 @@ bool GalWind<T>::compute() {
     else return compute_spherical();
     
 }
-template bool GalWind<float>::compute();
-template bool GalWind<double>::compute();
 
 
 template <class T>
@@ -170,8 +164,6 @@ bool GalWind<T>::compute_spherical() {
      
     return true;
 }
-template bool GalWind<float>::compute_spherical();
-template bool GalWind<double>::compute_spherical();
 
 
 template <class T>
@@ -224,8 +216,7 @@ bool GalWind<T>::compute_cylindrical() {
     // Starting progress bar
     bool verb = in->pars().isVerbose();
     if (verb) in->pars().setVerbosity(false);
-    ProgressBar bar(" Generating outflow model... ",false);
-    bar.setShowbar(in->pars().getShowbar());
+    ProgressBar bar(false,verb,in->pars().getShowbar());
     
     int nthreads = in->pars().getThreads();
 #pragma omp parallel num_threads(nthreads)
@@ -233,12 +224,12 @@ bool GalWind<T>::compute_cylindrical() {
     // Allocating Galmod objects for the two cylinders
     Model::Galmod<T> *con1 = new Model::Galmod<T>;
     Model::Galmod<T> *con2 = new Model::Galmod<T>;
-    if (verb) bar.init(par.NTOT);
+    bar.init(" Generating outflow model... ",par.NTOT);
     
 #pragma omp for schedule(dynamic)
     // Start main loop
     for (size_t k=1; k<(par.NTOT+1); k++){
-        if (verb) bar.update(k);
+        bar.update(k);
 
         // Assign distribution and kinematics to cylinder k
         float Rin  = par.DENSTYPE==2 ? (k-1)*width : 0;         // inner ring
@@ -298,14 +289,11 @@ bool GalWind<T>::compute_cylindrical() {
     delete con2;
 }
 
-    if (verb) bar.fillSpace(" Done.\n");
+    bar.fillSpace(" Done.\n");
     
     in->pars().setVerbosity(verb);
     return true;
 }
-template bool GalWind<float>::compute_cylindrical();
-template bool GalWind<double>::compute_cylindrical();
-
 
 
 template <class T>
@@ -332,8 +320,6 @@ bool GalWind<T>::smooth(bool scalefac) {
     
     
 }
-template bool GalWind<float>::smooth(bool);
-template bool GalWind<double>::smooth(bool);
 
 
 template <class T>
@@ -348,8 +334,6 @@ bool GalWind<T>::writeFITS(std::string fname, bool fullHead) {
     else return false;
 
 }
-template bool GalWind<float>::writeFITS(std::string, bool);
-template bool GalWind<double>::writeFITS(std::string, bool);
 
 
 template <class T>
@@ -370,8 +354,6 @@ bool GalWind<T>::writeMomentMaps() {
     }
     else return false;
 }
-template bool GalWind<float>::writeMomentMaps();
-template bool GalWind<double>::writeMomentMaps();
 
 
 template <class T>
@@ -391,8 +373,6 @@ bool GalWind<T>::writePV(std::string fname){
     else return false;
     
 }
-template bool GalWind<float>::writePV(std::string);
-template bool GalWind<double>::writePV(std::string);
 
 
 template <class T>
@@ -453,7 +433,7 @@ bool GalWind<T>::makePlots(){
         << "\ta = ax[k] \n"
         << "\tchan = int(k*(d.shape[0])/nc) \n"           
         << "\ttopl = d[chan]\n"     
-        << "\ta.tick_params(top='off',right='off',bottom='off',left='off',labelbottom='off',labelleft='off') \n"
+        << "\ta.tick_params(top=False,right=False,bottom=False,left=False,labelbottom=False,labelleft=False) \n"
         << "\tif np.all(topl<cont[0]): topl[:,:] = 0 \n"
         << "\ta.imshow(topl,origin='lower',cmap=cmap,norm=norm) \n"
         << "\ta.contour(topl,levels=cont,colors='#009999') \n"
@@ -481,24 +461,26 @@ bool GalWind<T>::makePlots(){
         << "labs  = ['Normalized $I$', r'$V_\\mathrm{LOS}$ (km/s)', r'$\\sigma$ (km/s)', 'Offset (arbitrary units)']\n"
         << "for k in range(len(topl)):\n"
         << "\ta = ax[k]\n"
-        << "\ta.tick_params(top='off',right='off',bottom='off',left='off',labelbottom='off',labelleft='off')\n"
+        << "\ta.tick_params(top=False,right=False,bottom=False,left=False,labelbottom=False,labelleft=False)\n"
         << "\ta.text(0.5,1.1,titls[k],transform=a.transAxes,ha='center',va='center')\n"
-        << "\tif k==1:\n"
+        << "\tif k==1:\n"            
         << "\t\txcen, ycen, phi = "<< x0 << "," << y0 << "," << PA << std::endl
-        << "\t\txx = np.arange(d.shape[-1])\n"
-        << "\t\tyy = np.tan(np.radians(phi))*(xx-xcen)+ycen \n"
-        << "\t\ta.plot(xx,yy,c='k',ls='--',alpha=0.7)\n\n"
+        << "\t\tif phi==90: a.axvline(xcen,c='k',ls='--',alpha=0.7) \n"
+        << "\t\telse: \n"
+        << "\t\t\txx = np.arange(d.shape[-1])\n"
+        << "\t\t\tyy = np.tan(np.radians(phi))*(xx-xcen)+ycen \n"
+        << "\t\t\ta.plot(xx,yy,c='k',ls='--',alpha=0.7)\n\n"
         
         << "\tif k==3: \n"
         << "\t\text = [-0.99,0.99 ,velos[0],velos[-1]]\n"
         << "\t\ta.set_ylim(ext[2]-2*abs(cdelt3_kms),ext[3]+2*abs(cdelt3_kms))\n"
-        << "\t\ta.tick_params(top='on',left='on',bottom='on',right='on',labelright='on')\n"
+        << "\t\ta.tick_params(top=True,left=True,bottom=True,right=True,labelright='on')\n"
         << "\t\ta.tick_params(axis='x', pad=7)\n"
         << "\t\ta.imshow(topl[k],origin='lower',cmap=cmaps[k],aspect='auto',extent=ext)\n"
         << "\t\ta.contour(topl[k],levels=cont,colors='#769BCB',extent=ext)\n"
         << "\t\ta.axhline(" << Vsys << ",c='k',ls='-',alpha=0.5)\n"
         << "\t\ta.axvline(0,c='k',ls='-',alpha=0.5)\n"
-        << "\t\ta.tick_params(labelbottom='on') \n"
+        << "\t\ta.tick_params(labelbottom=True) \n"
         << "\t\ta.text(1.25, 0.5,r'$V_\\mathrm{LOS}$ (km/s)',transform=a.transAxes,ha='center',va='center',rotation=90) \n"
         << "\telse:\n"
         << "\t\tm = a.imshow(topl[k],origin='lower',cmap=cmaps[k],aspect='auto',norm=norms[k]) \n"
@@ -514,11 +496,21 @@ bool GalWind<T>::makePlots(){
     pyf.close();
 
 #ifdef HAVE_PYTHON
+    // Now plotting everything
+    if (in->pars().isVerbose()) std::cout << " Producing " << randomAdjective(1) << " plots..." << std::flush;
     std::string cmd = "python "+in->pars().getOutfolder()+scriptname+" > /dev/null 2>&1";
-    return system(cmd.c_str());
+    int ret = system(cmd.c_str());
+    if (in->pars().isVerbose()) {
+        if (ret==0) std::cout << " Done." << std::endl;
+        else std::cout << " Something went wrong! Check pyscript.py in the output folder." << std::endl;
+    }
+    
 #endif
 
-    return false;
+    return true;
 }
-template bool GalWind<float>::makePlots();
-template bool GalWind<double>::makePlots();
+
+
+// Explicit instantiation of the class
+template class GalWind<float>;
+template class GalWind<double>;
