@@ -41,18 +41,14 @@ MomentMap<T>::MomentMap() {
     this->statsDefined   = false;
     
 }
-template MomentMap<short>::MomentMap(); 
-template MomentMap<int>::MomentMap(); 
-template MomentMap<long>::MomentMap(); 
-template MomentMap<float>::MomentMap(); 
-template MomentMap<double>::MomentMap(); 
 
 
 template <class T> 
-void MomentMap<T>::input (Cube<T> *c, int *Blo, int *Bhi) {
+void MomentMap<T>::input (Cube<T> *c, int *Blo, int *Bhi, bool *m) {
     
     in = c;
-    
+    mask = m;
+
     for (int i=0; i<3; i++) {
         blo[i] = Blo[i];
         bhi[i] = Bhi[i];
@@ -67,27 +63,17 @@ void MomentMap<T>::input (Cube<T> *c, int *Blo, int *Bhi) {
     this->arrayAllocated = true;
     this->par = c->pars();
 }
-template void MomentMap<short>::input(Cube<short>*,int*,int*); 
-template void MomentMap<int>::input(Cube<int>*,int*,int*); 
-template void MomentMap<long>::input(Cube<long>*,int*,int*); 
-template void MomentMap<float>::input(Cube<float>*,int*,int*); 
-template void MomentMap<double>::input(Cube<double>*,int*,int*); 
 
 
 template <class T> 
-void MomentMap<T>::input (Cube<T> *c) {
+void MomentMap<T>::input (Cube<T> *c, bool *m) {
     
     int blo[3] = {0,0,0};
     int bhi[3] = {c->AxesDim(0),c->AxesDim(1),c->AxesDim(2)};
     
-    input(c,blo,bhi);
+    input(c,blo,bhi,m);
     
 }
-template void MomentMap<short>::input(Cube<short>*); 
-template void MomentMap<int>::input(Cube<int>*); 
-template void MomentMap<long>::input(Cube<long>*); 
-template void MomentMap<float>::input(Cube<float>*); 
-template void MomentMap<double>::input(Cube<double>*); 
 
 
 template <class T>
@@ -102,11 +88,6 @@ void MomentMap<T>::SumMap (bool msk) {
     this->head.setBunit(in->Head().Bunit());
 
 }
-template void MomentMap<short>::SumMap(bool); 
-template void MomentMap<int>::SumMap(bool); 
-template void MomentMap<long>::SumMap(bool); 
-template void MomentMap<float>::SumMap(bool); 
-template void MomentMap<double>::SumMap(bool); 
 
 
 template <class T>
@@ -144,11 +125,6 @@ void MomentMap<T>::HIMassDensityMap (bool msk) {
         std::terminate();
     }
 }
-template void MomentMap<short>::HIMassDensityMap(bool); 
-template void MomentMap<int>::HIMassDensityMap(bool); 
-template void MomentMap<long>::HIMassDensityMap(bool); 
-template void MomentMap<float>::HIMassDensityMap(bool); 
-template void MomentMap<double>::HIMassDensityMap(bool); 
 
 
 template <class T>
@@ -168,8 +144,11 @@ void MomentMap<T>::storeMap(bool msk, int whichmap, std::string map_type) {
     }
     
     // Creating mask if it does not exist
-    if(msk && !in->MaskAll()) in->BlankMask();
-    
+    if(msk && mask==nullptr) {
+        if (!in->MaskAll()) in->BlankMask();
+        mask = in->Mask();
+    }
+
     std::string barstring;
     if (whichmap==0)      barstring = " Extracting intensity map ";
     else if (whichmap==1) barstring = " Extracting velocity map ";
@@ -183,19 +162,16 @@ void MomentMap<T>::storeMap(bool msk, int whichmap, std::string map_type) {
         map_Type = &MomentMap<T>::calculateMoments;
         barstring += "(MOMENT)... ";
     }
-
-    bool isVerbose = in->pars().isVerbose();
     
-    ProgressBar bar(barstring, true);
-    bar.setShowbar(in->pars().getShowbar());
+    ProgressBar bar(true,in->pars().isVerbose(),in->pars().getShowbar());
 
     int nthreads = in->pars().getThreads();    
 #pragma omp parallel num_threads(nthreads)
 {
-    if (isVerbose) bar.init(this->axisDim[0]);
+    bar.init(barstring,this->axisDim[0]);
 #pragma omp for
     for (int x=0; x<this->axisDim[0]; x++) {
-        if (isVerbose) bar.update(x+1);
+        bar.update(x+1);
         for (int y=0; y<this->axisDim[1]; y++) {
             this->array[x+y*this->axisDim[0]]=log(-1);
             double moms[3];
@@ -205,14 +181,9 @@ void MomentMap<T>::storeMap(bool msk, int whichmap, std::string map_type) {
     }
 }
     
-    if (isVerbose) bar.fillSpace("Done.\n");
+    bar.fillSpace("Done.\n");
     
 }
-template void MomentMap<short>::storeMap(bool,int,std::string);
-template void MomentMap<int>::storeMap(bool,int,std::string);
-template void MomentMap<long>::storeMap(bool,int,std::string);
-template void MomentMap<float>::storeMap(bool,int,std::string);
-template void MomentMap<double>::storeMap(bool,int,std::string);
 
 
 template <class T> 
@@ -232,7 +203,6 @@ void MomentMap<T>::RMSMap (float level, float sncut) {
         std::cout<< "MOMENT MAPS warning: cannot create new header.\n";
     }
     
-    bool isVerbose = in->pars().isVerbose();
     int nthreads = in->pars().getThreads();
     bool rob = in->pars().getFlagRobustStats();
     
@@ -240,15 +210,14 @@ void MomentMap<T>::RMSMap (float level, float sncut) {
     size_t xs = in->DimX(), ys = in->DimY(), zs = in->DimZ();
     
     // Progress bar
-    ProgressBar bar(" Computing RMS map... ", true);
-    bar.setShowbar(in->pars().getShowbar());
+    ProgressBar bar(true,in->pars().isVerbose(),in->pars().getShowbar());
     
 #pragma omp parallel num_threads(nthreads)
 {
-    if (isVerbose) bar.init(xs*ys);
+   bar.init(" Computing RMS map... ",xs*ys);
 #pragma omp for
     for (size_t xy=0; xy<xs*ys; xy++) {
-        if (isVerbose) bar.update(xy+1);
+        bar.update(xy+1);
             
         // Getting the spectrum at x,y pixel
         std::vector<float> sp(zs);
@@ -281,15 +250,9 @@ void MomentMap<T>::RMSMap (float level, float sncut) {
         this->array[xy] = orms;
     }
 }
-    if (isVerbose) bar.fillSpace("Done.\n");
-
+    bar.fillSpace("Done.\n");
 
 }
-template void MomentMap<short>::RMSMap(float,float); 
-template void MomentMap<int>::RMSMap(float,float); 
-template void MomentMap<long>::RMSMap(float,float); 
-template void MomentMap<float>::RMSMap(float,float); 
-template void MomentMap<double>::RMSMap(float,float); 
 
 
 template <class T> 
@@ -324,12 +287,6 @@ bool MomentMap<T>::setHead(int type) {
     return this->headDefined;
     
 }
-template bool MomentMap<short>::setHead(int); 
-template bool MomentMap<int>::setHead(int); 
-template bool MomentMap<long>::setHead(int); 
-template bool MomentMap<float>::setHead(int); 
-template bool MomentMap<double>::setHead(int); 
-
 
 
 template <class T>
@@ -347,8 +304,8 @@ bool MomentMap<T>::calculateMoments (size_t x, size_t y, bool msk, double *momen
         if (in->HeadDef()) vels[z] = AlltoVel<T>(in->getZphys(z), in->Head());
                 
         if (msk) {
-            num   += in->Array(npix)*vels[z]*in->Mask(npix);
-            denom += in->Array(npix)*in->Mask(npix);
+            num   += in->Array(npix)*vels[z]*mask[npix];
+            denom += in->Array(npix)*mask[npix];
         }
         else {
             num   += in->Array(npix)*vels[z];
@@ -359,6 +316,8 @@ bool MomentMap<T>::calculateMoments (size_t x, size_t y, bool msk, double *momen
     // If all pixels are masked return all NaNs
     if (denom==0) {
         moments[0] = moments[1] = moments[2] = log(-1);
+        delete [] spectrum;
+        delete [] vels;
         return true;
     }
     
@@ -374,7 +333,7 @@ bool MomentMap<T>::calculateMoments (size_t x, size_t y, bool msk, double *momen
     num = 0;
     for (int z=0; z<nsubs; z++) {
         long npix = in->nPix(x+blo[0],y+blo[1],z+blo[2]);
-        if (msk) num += in->Array(npix)*(vels[z]-moments[1])*(vels[z]-moments[1])*in->Mask(npix);
+        if (msk) num += in->Array(npix)*(vels[z]-moments[1])*(vels[z]-moments[1])*mask[npix];
         else num += in->Array(npix)*(vels[z]-moments[1])*(vels[z]-moments[1]);
     }
     
@@ -385,12 +344,6 @@ bool MomentMap<T>::calculateMoments (size_t x, size_t y, bool msk, double *momen
     delete [] vels;
     return true;
 }
-template bool MomentMap<short>::calculateMoments(size_t,size_t,bool,double*); 
-template bool MomentMap<int>::calculateMoments(size_t,size_t,bool,double*); 
-template bool MomentMap<long>::calculateMoments(size_t,size_t,bool,double*); 
-template bool MomentMap<float>::calculateMoments(size_t,size_t,bool,double*); 
-template bool MomentMap<double>::calculateMoments(size_t,size_t,bool,double*); 
-
 
 
 template <class T>
@@ -410,7 +363,7 @@ bool MomentMap<T>::fitSpectrum (size_t x, size_t y, bool msk, double *bestfitpar
     for (int z=0; z<nsubs; z++) {
         ww[z] = 1;
         spectrum[z] = in->Array(x,y,z);
-        if (msk) spectrum[z] *= in->Mask(x,y,z);
+        if (msk) spectrum[z] *= mask[in->nPix(x,y,z)];
         vels[z] = AlltoVel(in->getZphys(z),in->Head());
         // Finding spectrum maximum value and corresponding position
         if (spectrum[z]>smax) {
@@ -442,11 +395,6 @@ bool MomentMap<T>::fitSpectrum (size_t x, size_t y, bool msk, double *bestfitpar
     
     return true;
 }
-template bool MomentMap<short>::fitSpectrum(size_t,size_t,bool,double*); 
-template bool MomentMap<int>::fitSpectrum(size_t,size_t,bool,double*); 
-template bool MomentMap<long>::fitSpectrum(size_t,size_t,bool,double*); 
-template bool MomentMap<float>::fitSpectrum(size_t,size_t,bool,double*); 
-template bool MomentMap<double>::fitSpectrum(size_t,size_t,bool,double*); 
 
 
 template <class T>
@@ -566,3 +514,63 @@ template Image2D<long>* PositionVelocity (Cube<long>*,float,float,float);
 template Image2D<float>* PositionVelocity (Cube<float>*,float,float,float);
 template Image2D<double>* PositionVelocity (Cube<double>*,float,float,float);
 
+
+template <class T>
+std::vector< MomentMap<T> > getAllMoments(Cube<T> *c, bool usemask, bool *mask, string mtype) {
+
+    /// This function computes 0th, 1st and 2nd moment maps in a computationally
+    /// efficient way. Maps are stored and returned in a vector of MomentMap
+    /// instances.
+
+    std::vector< MomentMap<T> > allmaps(3);
+
+    // Creating mask if it does not exist
+    if(mask==nullptr && usemask) {
+        if (!c->MaskAll()) c->BlankMask();
+        mask = c->Mask();
+    }
+
+    // Initiliazing all moment maps
+    for (int i=0; i<3; i++) {
+        allmaps[i].input(c,mask);
+        allmaps[i].setHeadDef(allmaps[i].setHead(i));
+    }
+
+    ProgressBar bar(true,c->pars().isVerbose(),c->pars().getShowbar());
+
+    int nthreads = c->pars().getThreads();
+#pragma omp parallel num_threads(nthreads)
+{
+    bar.init(" Deriving kinematic maps... ",c->DimY());
+#pragma omp for
+    for (int y=0; y<c->DimY(); y++) {
+        bar.update(y+1);
+        for (int x=0; x<c->DimX(); x++) {
+            double moms[3];
+            if (mtype=="GAUSSIAN") allmaps[0].fitSpectrum(x,y,usemask,moms);
+            else allmaps[0].calculateMoments(x,y,usemask,moms);
+            allmaps[0].Array(x,y) = moms[0];
+            allmaps[1].Array(x,y) = moms[1];
+            allmaps[2].Array(x,y) = moms[2];
+        }
+    }
+}
+
+    bar.fillSpace("Done.\n");
+
+    return allmaps;
+}
+template std::vector< MomentMap<short> > getAllMoments(Cube<short>*,bool,bool*,std::string);
+template std::vector< MomentMap<int> > getAllMoments(Cube<int>*,bool,bool*,std::string);
+template std::vector< MomentMap<long> > getAllMoments(Cube<long>*,bool,bool*,std::string);
+template std::vector< MomentMap<float> > getAllMoments(Cube<float>*,bool,bool*,std::string);
+template std::vector< MomentMap<double> > getAllMoments(Cube<double>*,bool,bool*,std::string);
+
+
+
+// Explicit instantiation of the class
+template class MomentMap<short>;
+template class MomentMap<int>;
+template class MomentMap<long>;
+template class MomentMap<float>;
+template class MomentMap<double>;
