@@ -49,11 +49,6 @@ void Smooth3D<T>::defaults() {
     //func_psf = &Smooth3D<T>::defineBeam_Moffat;
     
 }
-template void Smooth3D<short>::defaults();
-template void Smooth3D<int>::defaults();
-template void Smooth3D<long>::defaults();
-template void Smooth3D<float>::defaults();
-template void Smooth3D<double>::defaults();
 
 
 template <class T>
@@ -61,11 +56,6 @@ Smooth3D<T>::Smooth3D() {
     
     defaults();
 }
-template Smooth3D<short>::Smooth3D();
-template Smooth3D<int>::Smooth3D();
-template Smooth3D<long>::Smooth3D();
-template Smooth3D<float>::Smooth3D();
-template Smooth3D<double>::Smooth3D();
 
 
 template <class T>
@@ -75,11 +65,6 @@ Smooth3D<T>::~Smooth3D() {
     if (blanksAllocated) delete [] blanks;
     if (confieAllocated) delete [] confie;
 }
-template Smooth3D<short>::~Smooth3D();
-template Smooth3D<int>::~Smooth3D();
-template Smooth3D<long>::~Smooth3D();
-template Smooth3D<float>::~Smooth3D();
-template Smooth3D<double>::~Smooth3D();
 
 
 template <class T>
@@ -87,11 +72,6 @@ Smooth3D<T>::Smooth3D(const Smooth3D<T> &s) {
 
     operator=(s);
 }
-template Smooth3D<short>::Smooth3D(const Smooth3D<short> &);
-template Smooth3D<int>::Smooth3D(const Smooth3D<int> &);
-template Smooth3D<long>::Smooth3D(const Smooth3D<long> &);
-template Smooth3D<float>::Smooth3D(const Smooth3D<float> &);
-template Smooth3D<double>::Smooth3D(const Smooth3D<double> &);
 
 
 template <class T>
@@ -153,11 +133,6 @@ Smooth3D<T>& Smooth3D<T>::operator=(const Smooth3D<T> &s) {
     
     return *this;
 }
-template Smooth3D<short>& Smooth3D<short>::operator=(const Smooth3D<short> &);
-template Smooth3D<int>& Smooth3D<int>::operator=(const Smooth3D<int> &);
-template Smooth3D<long>& Smooth3D<long>::operator=(const Smooth3D<long> &);
-template Smooth3D<float>& Smooth3D<float>::operator=(const Smooth3D<float> &);
-template Smooth3D<double>& Smooth3D<double>::operator=(const Smooth3D<double> &);
 
 
 template <class T>
@@ -210,11 +185,6 @@ void Smooth3D<T>::cubesmooth(Cube<T> *c) {
 
     smooth(c, Bhi, Blo, OB, NB);
 }
-template void Smooth3D<short>::cubesmooth(Cube<short>*);
-template void Smooth3D<int>::cubesmooth(Cube<int>*);
-template void Smooth3D<long>::cubesmooth(Cube<long>*);
-template void Smooth3D<float>::cubesmooth(Cube<float>*);
-template void Smooth3D<double>::cubesmooth(Cube<double>*);
 
 
 template <class T>
@@ -225,11 +195,6 @@ void Smooth3D<T>::smooth(Cube<T> *c, Beam Oldbeam, Beam Newbeam) {
     
     smooth(c, Bhi, Blo, Oldbeam, Newbeam);
 }
-template void Smooth3D<short>::smooth(Cube<short>*,Beam, Beam);
-template void Smooth3D<int>::smooth(Cube<int>*,Beam, Beam);
-template void Smooth3D<long>::smooth(Cube<long>*,Beam, Beam);
-template void Smooth3D<float>::smooth(Cube<float>*,Beam, Beam);
-template void Smooth3D<double>::smooth(Cube<double>*,Beam, Beam);
 
 
 template <class T>
@@ -310,11 +275,6 @@ void Smooth3D<T>::smooth(Cube<T> *c, int *Bhi, int *Blo, Beam Oldbeam, Beam Newb
     }
 
 }
-template void Smooth3D<short>::smooth(Cube<short>*,int*,int*,Beam, Beam);
-template void Smooth3D<int>::smooth(Cube<int>*,int*,int*,Beam, Beam);
-template void Smooth3D<long>::smooth(Cube<long>*,int*,int*,Beam, Beam);
-template void Smooth3D<float>::smooth(Cube<float>*,int*,int*,Beam, Beam);
-template void Smooth3D<double>::smooth(Cube<double>*,int*,int*,Beam, Beam);
 
 
 template <class T>
@@ -364,11 +324,6 @@ bool Smooth3D<T>::smooth(Cube<T> *c, Beam Oldbeam, Beam Newbeam, T *OldArray, T
     
     return true;
 }
-template bool Smooth3D<short>::smooth(Cube<short>*,Beam,Beam,short*,short*);
-template bool Smooth3D<int>::smooth(Cube<int>*,Beam,Beam,int*,int*);
-template bool Smooth3D<long>::smooth(Cube<long>*,Beam,Beam,long*,long*);
-template bool Smooth3D<float>::smooth(Cube<float>*,Beam,Beam,float*,float*);
-template bool Smooth3D<double>::smooth(Cube<double>*,Beam,Beam,double*,double*);
 
 
 template <class T>
@@ -477,11 +432,6 @@ bool Smooth3D<T>::defineBeam_Gaussian(Beam Oldbeam, Beam Newbeam) {
     
     return true;
 }
-template bool Smooth3D<short>::defineBeam_Gaussian(Beam,Beam);
-template bool Smooth3D<int>::defineBeam_Gaussian(Beam,Beam);
-template bool Smooth3D<long>::defineBeam_Gaussian(Beam,Beam);
-template bool Smooth3D<float>::defineBeam_Gaussian(Beam,Beam);
-template bool Smooth3D<double>::defineBeam_Gaussian(Beam,Beam);
 
 
 template <class T>
@@ -520,11 +470,6 @@ bool Smooth3D<T>::defineBeam_Moffat(Beam Oldbeam, Beam Newbeam) {
     
     return true;
 }
-template bool Smooth3D<short>::defineBeam_Moffat(Beam,Beam);
-template bool Smooth3D<int>::defineBeam_Moffat(Beam,Beam);
-template bool Smooth3D<long>::defineBeam_Moffat(Beam,Beam);
-template bool Smooth3D<float>::defineBeam_Moffat(Beam,Beam);
-template bool Smooth3D<double>::defineBeam_Moffat(Beam,Beam);
 
 
 template <class T>
@@ -537,9 +482,7 @@ bool Smooth3D<T>::calculate(T *OldArray, T *NewArray) {
     
     long size = (NdatX+NconX-1)*(NdatY+NconY-1);
         
-    bool verb = in->pars().isVerbose();
-    ProgressBar bar(" Smoothing... ",false);
-    bar.setShowbar(in->pars().getShowbar());
+    ProgressBar bar(false,in->pars().isVerbose(),in->pars().getShowbar());
 
     if (!usescalefac) scalefac=1.0;
     
@@ -547,12 +490,12 @@ bool Smooth3D<T>::calculate(T *OldArray, T *NewArray) {
 
 #pragma omp parallel num_threads(nthreads)
 {
-    if (verb) bar.init(NdatZ);
+    bar.init(" Smoothing... ",NdatZ);
     T *beforeCON = new T[size];
     T *afterCON  = new T[size];
 #pragma omp for
     for (int z=0; z<NdatZ; z++) {
-        if (verb) bar.update(z+1);  
+        bar.update(z+1);
         for (int x=0; x<(NdatX+NconX-1); x++) {
             for (int y=0; y<(NdatY+NconY-1); y++) {
                 long nPix = x+y*(NdatX+NconX-1);
@@ -586,15 +529,10 @@ bool Smooth3D<T>::calculate(T *OldArray, T *NewArray) {
     delete [] beforeCON;
     delete [] afterCON; 
 }
-    if (verb) bar.fillSpace("OK.\n");    
+    bar.fillSpace("OK.\n");
     
     return true;
 }
-template bool Smooth3D<short>::calculate(short*,short*);
-template bool Smooth3D<int>::calculate(int*,int*);
-template bool Smooth3D<long>::calculate(long*,long*);
-template bool Smooth3D<float>::calculate(float*,float*);
-template bool Smooth3D<double>::calculate(double*,double*);
 
 
 template <class T>
@@ -605,24 +543,20 @@ bool Smooth3D<T>::calculatefft(T *OldArray, T *NewArray) {
         return false;
     }
     
-        
-    bool verb = in->pars().isVerbose();
-    ProgressBar bar(" Smoothing... ",false);
-    bar.setShowbar(in->pars().getShowbar());
+    ProgressBar bar(false,in->pars().isVerbose(),in->pars().getShowbar());
     
     if (!usescalefac) scalefac=1.0;
     int nthreads = in->pars().getThreads();
 
 #pragma omp parallel num_threads(nthreads)
 {
-    if (verb) bar.init(NdatZ);
+    bar.init(" Smoothing... ",NdatZ);
     Conv2D conv_fft;
     init_Conv2D(conv_fft,LINEAR_SAME, NdatX, NdatY, NconX, NconY);
     double *beforeCON = new double[NdatX*NdatY];
 #pragma omp for
     for (int z=0; z<NdatZ; z++) {
-
-        if (verb) bar.update(z+1);  
+        bar.update(z+1);
         for (int x=0; x<NdatX; x++) {
             for (int y=0; y<NdatY; y++) {
                 long nPix = x+y*NdatX;
@@ -646,15 +580,10 @@ bool Smooth3D<T>::calculatefft(T *OldArray, T *NewArray) {
     clear_Conv2D(conv_fft);
 }
 
-    if (verb) bar.fillSpace("Done.\n");
+    bar.fillSpace("Done.\n");
 
     return true;
 }
-template bool Smooth3D<short>::calculatefft(short*,short*);
-template bool Smooth3D<int>::calculatefft(int*,int*);
-template bool Smooth3D<long>::calculatefft(long*,long*);
-template bool Smooth3D<float>::calculatefft(float*,float*);
-template bool Smooth3D<double>::calculatefft(double*,double*);
 
 
 template <class T>  
@@ -724,11 +653,6 @@ int Smooth3D<T>::Convolve(double *cfie, int ncx, int ncy, T *dat1, T *dat2, int
   
    return ier;
 }
-template int Smooth3D<short>::Convolve(double*,int,int,short*,short*,int,int);
-template int Smooth3D<int>::Convolve(double*,int,int,int*,int*,int,int);
-template int Smooth3D<long>::Convolve(double*,int,int,long*,long*,int,int);
-template int Smooth3D<float>::Convolve(double*,int,int,float*,float*,int,int);
-template int Smooth3D<double>::Convolve(double*,int,int,double*,double*,int,int);
 
 
 template <class T>
@@ -792,11 +716,6 @@ bool Smooth3D<T>::Convpars() {
     return true;
  
 }
-template bool Smooth3D<short>::Convpars();
-template bool Smooth3D<int>::Convpars();
-template bool Smooth3D<long>::Convpars();
-template bool Smooth3D<float>::Convpars();
-template bool Smooth3D<double>::Convpars();
 
 
 template <class T>
@@ -845,8 +764,8 @@ bool Smooth3D<T>::Fillgauss2d(Beam varbeam, float ampl, bool norm, int &nconx, i
     int Xmax = round(x/gridspac[0]); 
     int Ymax = round(y/gridspac[1]); 
 
-    int nconX = 2*Xmax+1;    
-    int nconY = 2*Ymax+1; 
+    size_t nconX = 2*Xmax+1;    
+    size_t nconY = 2*Ymax+1; 
 
     if ((nconX*nconY)>maxconv) {
         std::cout << "SMOOTH error: Convolution function too big for buffer.\n";
@@ -883,19 +802,13 @@ bool Smooth3D<T>::Fillgauss2d(Beam varbeam, float ampl, bool norm, int &nconx, i
    
 
     if (norm) {
-        for (int i=0; i<(nconX*nconY); i++) {
+        for (size_t i=0; i<(nconX*nconY); i++) {
             cfie[i] = cfie[i]/totalarea;
         }
     }
     
     return true;   
 }
-template bool Smooth3D<short>::Fillgauss2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<int>::Fillgauss2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<long>::Fillgauss2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<float>::Fillgauss2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<double>::Fillgauss2d(Beam,float,bool,int&,int&,double*);
-
 
 
 template <class T>
@@ -932,8 +845,8 @@ bool Smooth3D<T>::FillMoffat2d(Beam varbeam, float ampl, bool norm, int &nconx,
     int Xmax = round(r/gridspac[0]); 
     int Ymax = round(r/gridspac[1]); 
 
-    int nconX = 2*Xmax+1;    
-    int nconY = 2*Ymax+1; 
+    size_t nconX = 2*Xmax+1;    
+    size_t nconY = 2*Ymax+1; 
 
     if ((nconX*nconY)>maxconv) {
         std::cout << "SMOOTH error: Convolution function too big for buffer.\n";
@@ -942,7 +855,7 @@ bool Smooth3D<T>::FillMoffat2d(Beam varbeam, float ampl, bool norm, int &nconx,
         nconx = nconX;
         ncony = nconY;
     }
-    
+
     double sigma = fwhm/(2*sqrt(pow(2,1/pidx)-1.));
         
     double totalarea = 0;
@@ -964,18 +877,13 @@ bool Smooth3D<T>::FillMoffat2d(Beam varbeam, float ampl, bool norm, int &nconx,
     }
    
     if (norm) {
-        for (int i=0; i<(nconX*nconY); i++) {
+        for (size_t i=0; i<(nconX*nconY); i++) {
             cfie[i] = cfie[i]/totalarea;
         }
     }
     
     return true;   
 }
-template bool Smooth3D<short>::FillMoffat2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<int>::FillMoffat2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<long>::FillMoffat2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<float>::FillMoffat2d(Beam,float,bool,int&,int&,double*);
-template bool Smooth3D<double>::FillMoffat2d(Beam,float,bool,int&,int&,double*);
 
 
 template <class T> 
@@ -1020,7 +928,7 @@ void Smooth3D<T>::fitswrite() {
     
     int factor = floor(in->pars().getFactor());
     if (in->pars().getflagReduce() && factor>1) {
-        Cube<T> *red = out->Reduce(factor); 
+        Cube<T> *red = out->Reduce(factor,"spatial"); 
         red->fitswrite_3d(name.c_str(),true);
         delete red;
     }
@@ -1033,65 +941,89 @@ void Smooth3D<T>::fitswrite() {
     }
     delete out;
 }
-template void Smooth3D<short>::fitswrite();
-template void Smooth3D<int>::fitswrite();
-template void Smooth3D<long>::fitswrite();
-template void Smooth3D<float>::fitswrite();
-template void Smooth3D<double>::fitswrite();
 
 
 /////////////////////////////////////////////////////////////////////////////////////
-/// Function defintion for Hanning smoothing class
+/// Function defintion for SpectralSmooth class
 /////////////////////////////////////////////////////////////////////////////////////
 
 template <class T>
-void Hanning3D<T>::compute(Cube<T> *in) {
+SpectralSmooth3D<T>::SpectralSmooth3D(std::string wtype, size_t wsize) {
+     windowtype = makeupper(wtype);
+     windowsize = wsize;
+}
+
+
+template <class T>
+void SpectralSmooth3D<T>::smooth(Cube<T> *in) {
     
-    // Performs Hanning smoothing on each spectrum of a datacube
-    if (in->pars().isVerbose()) std::cout << " Hanning smoothing..." << std::flush;
-    this->compute(in->Array(),in->DimX(),in->DimY(),in->DimZ());
+    // Performs smoothing on each spectrum of a datacube
+    if (in->pars().isVerbose()) std::cout << " Spectral smoothing (" << windowtype << ") ..." << std::flush;
+    this->smooth(in->Array(),in->DimX(),in->DimY(),in->DimZ(),in->pars().getThreads());
     if (in->pars().isVerbose()) std::cout << " Done!" << std::endl;
     
 }
-template void Hanning3D<float>::compute(Cube<float>*);
-template void Hanning3D<double>::compute(Cube<double>*);
 
 
 template <class T>
-void Hanning3D<T>::compute(T *inarray, size_t xsize, size_t ysize, size_t zsize) {
+void SpectralSmooth3D<T>::smooth(T *inarray, size_t xsize, size_t ysize, size_t zsize, int nthreads) {
     
-    // Performs Hanning smoothing on each spectrum of a 3D array
+    // Performs Spectral smoothing on each spectrum of a 3D array
+    if (arrayAllocated) delete [] array;
     array = new T[xsize*ysize*zsize];
     arrayAllocated = true;
-    for (size_t x=0; x<xsize; x++) {
-        for (size_t y=0; y<ysize; y++) {
-            T *spec = new T[zsize];
-            for (size_t z=0; z<zsize; z++) spec[z] = inarray[x+y*xsize+z*ysize*xsize];
-            T* hann = HanningSmoothing<T>(spec,zsize,window);
-            for (size_t z=0; z<zsize; z++) array[x+y*xsize+z*ysize*xsize] = hann[z];
-            delete [] hann;
-            delete [] spec;
-        }
+
+#pragma omp parallel for num_threads(nthreads)
+    for (size_t i=0; i<xsize*ysize; i++) {
+        T *spec = new T[zsize];
+        for (size_t z=0; z<zsize; z++) spec[z] = inarray[i+z*ysize*xsize];
+        T* specsmooth = Smooth1D<T>(spec,zsize,windowtype,windowsize);
+        for (size_t z=0; z<zsize; z++) array[i+z*ysize*xsize] = specsmooth[z];
+        delete [] specsmooth;
+        delete [] spec;
     }
 }
-template void Hanning3D<float>::compute(float*,size_t,size_t,size_t);
-template void Hanning3D<double>::compute(double*,size_t,size_t,size_t);
 
 
 template <class T> 
-void Hanning3D<T>::fitswrite(Cube<T> *templ) {
+void SpectralSmooth3D<T>::fitswrite(Cube<T> *templ, std::string outname) {
     
     Cube<T> *out = new Cube<T>(*templ);
-    out->Head().Keys().push_back("HISTORY BBAROLO HANNING SMOOTHING: Hanning smoothed with window "+to_string(window)+" channels");
+    out->Head().Keys().push_back("HISTORY BBAROLO SPECTRAL SMOOTHING: "+windowtype+" window of size "+to_string(windowsize)+" channels");
     for (size_t i=0; i<out->NumPix(); i++) out->Array()[i] = array[i];
-    std::string outname = templ->pars().getOutfolder()+templ->Head().Name()+"_h"+to_string(window)+".fits";
-    out->fitswrite_3d(outname.c_str(),true);
-    if (templ->pars().isVerbose()) std::cout << " Smoothed datacube written in " << outname << std::endl;
+    if (outname=="") {
+        outname = templ->pars().getOutfolder()+templ->Head().Name()+"_h"+to_string(windowsize);
+        if (templ->pars().getflagReduce()) outname += "_red";
+        outname += ".fits";
+    }
+
+    if (templ->pars().getflagReduce() && windowsize>1) {
+        //int bins = windowtype=="HANNING" ? (windowsize+1)/2 : windowsize;
+        int bins = windowsize;
+        Cube<T> *red = out->Reduce(bins,"spectral");
+        red->fitswrite_3d(outname.c_str(),true);
+        delete red;
+    }
+    else {
+        T minn,maxx;
+        findMinMax<T>(out->Array(), out->NumPix(), minn, maxx);
+        out->Head().setDataMax(double(maxx));
+        out->Head().setDataMin(double(minn));
+        out->fitswrite_3d(outname.c_str(),true);
+    }
+
+    if (templ->pars().isVerbose()) std::cout << " Spectrally-smoothed datacube written in " << outname << std::endl;
     delete out;
 }
-template void Hanning3D<float>::fitswrite(Cube<float>*);
-template void Hanning3D<double>::fitswrite(Cube<double>*);
 
 
+// Explicit instantiation of the classes
+template class Smooth3D<short>;
+template class Smooth3D<int>;
+template class Smooth3D<long>;
+template class Smooth3D<float>;
+template class Smooth3D<double>;
+template class SpectralSmooth3D<float>;
+template class SpectralSmooth3D<double>;
 
 
