@@ -19,6 +19,7 @@
 
 #include <iostream>
 #include <cstring>
+#include <cstdlib>
 #include <cfloat> 
 #include <Tasks/spacepar.hh>
 #include <Utilities/utils.hh>
@@ -83,8 +84,6 @@ Spacepar<T>::Spacepar(Cube<T> *c) : Galfit<T>::Galfit(c) {
     
     parspAll = false;
 }
-template Spacepar<float>::Spacepar(Cube<float>*);
-template Spacepar<double>::Spacepar(Cube<double>*);
 
 
 template <class T>
@@ -130,15 +129,14 @@ void Spacepar<T>::calculate() {
         T p1min=0, p2min=0;
         T funmin=FLT_MAX;
         
-        ProgressBar bar("   Calculating models...",true);
-        bar.setShowbar(Galfit<T>::in->pars().getShowbar());
+        ProgressBar bar(true,verb,this->in->pars().getShowbar());
 
         int nthreads = Galfit<T>::in->pars().getThreads();
         // Starting loops over parameters
 #pragma omp parallel num_threads(nthreads) shared(funmin,p1min,p2min)
 {
         // NB: THIS LOOP IN PARALLEL CAUSES SOMETIMES SEGFAULT (in fftw_execute)
-        if(verb) bar.init(p1_nsteps);
+        bar.init("   Calculating models...",p1_nsteps);
 #pragma omp for 
         for (int i=0; i<p1_nsteps; i++) {
             if (verb) bar.update(i+1);
@@ -251,8 +249,6 @@ void Spacepar<T>::calculate() {
     delete [] p1steps;
     delete [] p2steps;
 }
-template void Spacepar<float>::calculate();
-template void Spacepar<double>::calculate();
 
 
 template <class T>
@@ -303,8 +299,6 @@ void Spacepar<T>::plotmin_Gnuplot (int nr) {
     gp.end();
 #endif  
 }
-template void Spacepar<float>::plotmin_Gnuplot(int);
-template void Spacepar<double>::plotmin_Gnuplot(int);
 
 
 template <class T>
@@ -312,24 +306,34 @@ void Spacepar<T>::plotAll_Python() {
     
     if (!parspAll) return;
     
-    bool verb = Galfit<T>::in->pars().isVerbose();
+    bool verb = this->in->pars().isVerbose();
     
     // Writing fitsfile with parameter space
-    std::string outfold = Galfit<T>::in->pars().getOutfolder();
-    std::string object  = Galfit<T>::in->Head().Name();
+    std::string outfold = this->in->pars().getOutfolder();
+    std::string object  = this->in->Head().Name();
     std::string fname   = outfold+"spacepar_"+object+".fits";
+    
+    // Writing parameter space as variation from minimum
+    size_t ms = parspace->DimX()*parspace->DimY();
+    for (int z=0; z<parspace->DimZ(); z++) {
+        float minval = *min_element(&parspace->Array()[z*ms], &parspace->Array()[z*ms]+ms);
+        for (int i=0; i<ms; i++) parspace->Array(z*ms+i) = fabs(parspace->Array(z*ms+i)-minval)/minval;
+    }
     parspace->fitswrite_3d(fname.c_str());
     
     // Writing python script for output plots
+    mkdirp((this->in->pars().getOutfolder()+"plotscripts/").c_str());
     std::string scriptname = "spacepar.py";
     std::string fout = outfold+"spacepar_"+object+".pdf";
-    std::ofstream py_file((outfold+scriptname).c_str());
+    std::ofstream py_file((outfold+"plotscripts/"+scriptname).c_str());
     
     py_file << "# A script to produce output plots for SPACEPAR task.\n"
             << "import numpy as np\n" 
             << "import matplotlib\n"
             << "import matplotlib.pyplot as plt\n"
-            << "from astropy.io import fits\n" 
+            << "from astropy.io import fits\n"
+            << "from astropy.convolution import convolve,Gaussian2DKernel\n"
+                
             << "matplotlib.rc('xtick',direction='in')\n" 
             << "matplotlib.rc('ytick',direction='in')\n" 
             << "matplotlib.rc('font',family='sans-serif',serif='Helvetica',size=10)\n\n"
@@ -365,13 +369,16 @@ void Spacepar<T>::plotAll_Python() {
             << "for i in range (nrad):\n"
             << "\tnr = int(i/ncols) + 1\n"
             << "\tnc = i - (nr-1)*ncols + 1\n"   
-            << "\ttoplot = d[i]\n"
+            << "\ttoplot = 100*d[i]\n"
+            << "\ttoplot_sm = convolve(toplot,Gaussian2DKernel(x_stddev=1),boundary='extend')\n"
             << "\ta = np.unravel_index(np.argmin(toplot),toplot.shape)\n"
             << "\tp1min, p2min = p1ran[a[1]], p2ran[a[0]]\n"
             << "\tax = axis[i]\n"
             << "\tax.set_xlim(ext[0],ext[1])\n"
             << "\tax.set_ylim(ext[2],ext[3])\n"
             << "\tax.imshow(toplot,origin='lower',extent=ext,aspect='auto',cmap=cmap)\n"
+            << "\tcs = ax.contour(toplot_sm,levels=[2,5,10,30],colors='w',origin='lower',extent=ext)\n"
+            << "\tax.clabel(cs, cs.levels, inline=True, fmt='%.0f', fontsize=9)\n"
             << "\tax.plot(p1min,p2min,'x',mew=2,ms=8,c='w')\n"
             << "\tradstr = 'R = %.2f %s'%(rings[i],ru)\n"
             << "\tminstr = 'min = (%.1f %s, %.1f %s)'%(p1min,p1u,p2min,p2u)\n"
@@ -382,12 +389,15 @@ void Spacepar<T>::plotAll_Python() {
             << "\t\tax.set_xlabel(p1+' ('+p1u+')')\n\n"
                 
             << "fig.savefig('"+fout+"',bbox_inches='tight')\n";
-    
+
+    py_file.close();
+
+
 #ifdef HAVE_PYTHON
-    std::string cmd = "python "+outfold+scriptname;//+" > /dev/null 2>&1";
-    if (Galfit<T>::in->pars().getFlagPlots()) {
+    std::string cmd = "python "+outfold+"plotscripts/"+scriptname +" > /dev/null 2>&1 ";
+    if (this->in->pars().getFlagPlots()) {
         if (verb) std::cout << "\n\nProducing " << randomAdjective(1) << " parameter space plots..." << std::flush;
-        int ret = system(cmd.c_str());
+        int ret = std::system(cmd.c_str());
         if (verb) {
             if (ret==0) std::cout << " Done.\n";
             else std::cout << " Something went wrong! Check spacepar.py in the output folder.\n";
@@ -396,8 +406,6 @@ void Spacepar<T>::plotAll_Python() {
 #endif
     
 }
-template void Spacepar<float>::plotAll_Python();
-template void Spacepar<double>::plotAll_Python();
 
 
 template <class T>
@@ -439,6 +447,8 @@ void Spacepar<T>::setOutSpacepar(int p1_nsteps, int p2_nsteps) {
     parspace->saveHead(h);
     parspAll = true;
 }
-template void Spacepar<float>::setOutSpacepar(int,int);
-template void Spacepar<double>::setOutSpacepar(int,int);
 
+
+// Explicit instantiation of the class
+template class Spacepar<float>;
+template class Spacepar<double>;
