@@ -40,6 +40,7 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <Utilities/allocator.hpp>
 
 
 std::string get_currentpath() {
@@ -54,17 +55,6 @@ double KpcPerArc(double d) {return 2*d*tan(M_PI/2/180)/3.6;}
 double VeltoDist(double vsys) {return vsys/70.;}
 double RedtoDist(double redshift) {return redshift*299792.458/70.;}
 
-template <class Type> 
-bool isNaN (Type n) {
-    volatile Type d = n; 
-    return d != d;
-}
-template bool isNaN (short);
-template bool isNaN (int);
-template bool isNaN (long);
-template bool isNaN (float);
-template bool isNaN (double);
-
 
 bool fexists(std::string filename) {std::ifstream ifile(filename.c_str()); return ifile.good();}
 
@@ -92,120 +82,6 @@ bool mkdirp(const char* path, mode_t mode) {
 }
 
 
-void FitsWrite_2D (const char *filename, float *image, long xsize, long ysize) {
-    
-    fitsfile *fptr;
-    int status, numAxes = 2;
-    long firstPix = 1, numPix;
-    long dimAxes[2] = {xsize, ysize};
-    numPix = xsize*ysize;
-    
-    remove (filename);
-    
-    //Create the new file
-    status = 0;
-    if (fits_create_file (&fptr, filename, &status)) {
-        fits_report_error (stderr, status);
-        }
-    
-    //Create the primary array image    
-    if (fits_create_img (fptr, FLOAT_IMG, numAxes, dimAxes, &status)){
-        fits_report_error (stderr, status);
-    }
-    
-    
-    if (fits_write_img (fptr, TFLOAT, firstPix, numPix, image, &status)){
-        fits_report_error (stderr, status);
-    }
-    
-    // Close the FITS File
-    if (fits_close_file(fptr, &status)){
-        fits_report_error(stderr, status);
-    }
-    
-}
-
-
-void FitsWrite_2D (const char *filename, double *image, long xsize, long ysize) {
-
-    fitsfile *fptr;
-    int status, numAxes = 2;
-    long firstPix = 1, numPix;
-    long dimAxes[2] = {xsize, ysize};
-    numPix = xsize*ysize;
-
-    remove (filename);
-
-    //Create the new file
-    status = 0;
-    if (fits_create_file (&fptr, filename, &status)) {
-        fits_report_error (stderr, status);
-        }
-
-    //Create the primary array image
-    if (fits_create_img (fptr, DOUBLE_IMG, numAxes, dimAxes, &status)){
-        fits_report_error (stderr, status);
-    }
-
-
-    if (fits_write_img (fptr, TDOUBLE, firstPix, numPix, image, &status)){
-        fits_report_error (stderr, status);
-    }
-
-    // Close the FITS File
-    if (fits_close_file(fptr, &status)){
-        fits_report_error(stderr, status);
-    }
-
-}
-
-
-void FitsWrite_3D (const char *outfile, float *outcube, long *dimAxes) {
-    
-    fitsfile *fptr;
-    int status = 0;      
-    long  fpixel = 1, naxis = 3, nelements;
-    long dnaxes[3] = {dimAxes[0], dimAxes[1], dimAxes[2]};
-    long naxes[3] = {dimAxes[2], dimAxes[1], dimAxes[0]};
- 
-    remove(outfile);             
-    status = 0;   
-    fits_create_file(&fptr, outfile, &status);  
-    fits_create_img(fptr, FLOAT_IMG, naxis, dnaxes, &status);
-        
-    nelements = naxes[0]*naxes[1]*naxes[2]; 
-    
-    fits_write_img(fptr, TFLOAT, fpixel, nelements, outcube, &status);
-
-    fits_close_file(fptr, &status);      
-  
-    fits_report_error(stderr, status);  
-}
-
-
-void FitsWrite_3D (const char *outfile, short *outcube, long *dimAxes) {
-    
-    fitsfile *fptr;
-    int status = 0;      
-    long  fpixel = 1, naxis = 3, nelements;
-    long dnaxes[3] = {dimAxes[0], dimAxes[1], dimAxes[2]};
-    long naxes[3] = {dimAxes[2], dimAxes[1], dimAxes[0]};
- 
-    remove(outfile);             
-    status = 0;   
-    fits_create_file(&fptr, outfile, &status);  
-    fits_create_img(fptr, SHORT_IMG, naxis, dnaxes, &status);
-        
-    nelements = naxes[0]*naxes[1]*naxes[2]; 
-    
-    fits_write_img(fptr, TSHORT, fpixel, nelements, outcube, &status);
-
-    fits_close_file(fptr, &status);      
-  
-    fits_report_error(stderr, status);  
-}
-
-
 template <class T> 
 T AlltoVel (T in, Header &h) {
     
@@ -228,7 +104,7 @@ T AlltoVel (T in, Header &h) {
     }
     else if (cunit2=="mum" || cunit2=="um" || cunit2=="micron" ||
              cunit2=="a" || cunit2=="ang"  || cunit2=="angstrom") {          // Micron or Angstrom for Hi-Z
-        //int z_cent = floor(h.DimAx(2)/2.)-1;        
+        //int z_cent = floor(h.DimAx(2)/2.)-1;
         double z_cent = h.Crpix(2)-1;
         double restw = h.Wave0(), reds = h.Redshift();
         if (restw!=-1) {
@@ -506,24 +382,26 @@ double arcsconv(std::string cunit) {
     else if (Cunit=="arcmin" || Cunit=="arcm") return 60.;
     else if (Cunit=="arcsec" || Cunit=="arcs") return 1.;
     else {
-        std::cout << "Conversion error (unknown CUNIT for RA-DEC): ";
-        std::cout << "cannot convert to ARCSEC.\n";
-        std::cout << cunit;
-        std::terminate(); 
+        std::string errmsg = "Conversion error (unknown CUNIT for RA-DEC): " + cunit + ".";
+        errmsg += " Cannot convert to ARCSEC.\n";
+        //throw errmsg;
+        std::cerr << errmsg;
+        std::terminate();
     }
 }
 
 
 double degconv(std::string cunit) {
 
-    std::string Cunit = makelower(cunit);
-    if (Cunit=="degree" || Cunit=="degrees" || Cunit=="deg") return 1.;
-    else if (Cunit=="arcmin" || Cunit=="arcm") return 1./60.;
-    else if (Cunit=="arcsec" || Cunit=="arcs") return 1./3600.;
+    std::string c = makelower(cunit);
+    if (c.find("deg")!=std::string::npos) return 1.;
+    else if (c.find("arcm")!=std::string::npos) return 1./60.;
+    else if (c.find("arcs")!=std::string::npos) return 1./3600.;
     else {
-        std::cout << "Conversion error (unknown CUNIT for RA-DEC): ";
-        std::cout << "cannot convert to DEGREE.\n";
-        std::cout << cunit;
+        std::string errmsg = "Conversion error (unknown CUNIT for RA-DEC): " + cunit + ".";
+        errmsg += " Cannot convert to DEGREE.\n";
+        //throw errmsg;
+        std::cerr << errmsg;
         std::terminate();
     }
 }
@@ -1017,42 +895,63 @@ template double* SimulateNoise(double,size_t);
 
 
 template <class T>
-T* HanningSmoothing(T *inarray, size_t npts, size_t hanningSize) {
+T* Smooth1D(T *inarray,size_t npts,std::string windowType,size_t windowSize) {
     
-    // Performs Hanning smoothing on a single 1D array
+    // Performs smoothing on a single 1D array. Accepted windows are below 
     
-    T *newarray = new T[npts];
-    
-    if(hanningSize%2==0){ 
-      std::cerr << "Hanning: need an odd number for the size. "
-  	      << "Changing "<< hanningSize << " to " << hanningSize+1<<".\n";
-      hanningSize++;
+    bool known_window = windowType=="HANNING" || windowType=="HANNING2" ||
+                        windowType=="BOXCAR"  || windowType=="TOPHAT"   || 
+                        windowType=="FLATTOP" || windowType=="BARTLETT" ||
+                        windowType=="WELCH"   || windowType=="BLACKMAN";
+    if (!known_window) {
+         std::cerr << "Smoothing 1D: window type unknown "
+                   << "Changing "<< windowType << " to \"HANNING\" .\n";
+         windowType = "HANNING";
+    }
+    // Check window size and type
+    if(windowSize%2==0){ 
+      std::cerr << "Smoothing 1D: need an odd number for the window size. "
+  	            << "Changing "<< windowSize << " to " << windowSize+1<<".\n";
+      windowSize++;
+    }
+   
+    // Defining coefficients for smoothing
+    double *coeff = new double[windowSize];
+    float scale = (windowSize+1.)/2.;
+    float N = windowSize-1;
+    float sum = 0;
+    for(size_t j=0; j<windowSize; j++) {
+        float x = j-(windowSize-1)/2.;
+        if (windowType=="HANNING")            // Hanning used in radio-astronomy
+            coeff[j] = 0.5+0.5*cos(x*M_PI/scale);
+        else if (windowType=="HANNING2")      // Classical Hanning window
+            coeff[j] = 0.5-0.5*cos(j*2*M_PI/N);
+        else if (windowType=="BARTLETT")      // Triangular window
+            coeff[j] = 1-fabs((j-N/2.)/(N/2.));
+        else if (windowType=="WELCH")
+            coeff[j] = 1-std::pow((j-N/2.)/(N/2.),2);
+        else if (windowType=="BLACKMAN") 
+            coeff[j] = 0.42659-0.49656*cos(j*2*M_PI/N)+0.076849*cos(j*4*M_PI/N);
+        else if (windowType=="FLATTOP")
+            coeff[j] =  0.21557895-0.416631580*cos(j*2*M_PI/N)+0.277263158*cos(j*4*M_PI/N)
+                                  -0.083578957*cos(j*6*M_PI/N)+0.006947368*cos(j*8*M_PI/N);
+        else coeff[j] = 1.;
+        sum += coeff[j]; 
     }
     
-    float scale = (hanningSize+1.)/2.;
+    // Smooth
+    T *newarray = new T[npts];
     for(size_t i=0; i<npts; i++){
         newarray[i] = 0.;
-        for(size_t j=0; j<hanningSize; j++){
-            float x = j-(hanningSize-1)/2.;
-            double coeff = (0.5+0.5*cos(x*M_PI/scale))/scale;
-            if((i+x>0)&&(i+x<npts)) newarray[i] += coeff*inarray[i+int(x)];
+        for(size_t j=0; j<windowSize; j++){
+            float x = j-(windowSize-1)/2.;
+            if((i+x>0)&&(i+x<npts)) newarray[i] += coeff[j]/sum*inarray[i+int(x)];
         }
     }
+    delete [] coeff;
     return newarray;
 }
-template float* HanningSmoothing(float*,size_t,size_t);
-template double* HanningSmoothing(double*,size_t,size_t);
+template float* Smooth1D(float*,size_t,std::string,size_t);
+template double* Smooth1D(double*,size_t,std::string,size_t);
 
 
-
-template <> int selectBitpix<short>() {return SHORT_IMG;}
-template <> int selectBitpix<int>() {return SHORT_IMG;}
-template <> int selectBitpix<long>() {return LONG_IMG;}
-template <> int selectBitpix<float>() {return FLOAT_IMG;}
-template <> int selectBitpix<double>() {return DOUBLE_IMG;}
-
-template <> int selectDatatype<short>() {return TSHORT;}
-template <> int selectDatatype<int>() {return TINT;}
-template <> int selectDatatype<long>() {return TLONG;}
-template <> int selectDatatype<float>() {return TFLOAT;}
-template <> int selectDatatype<double>() {return TDOUBLE;}
